@@ -4,10 +4,12 @@
 
 #include "system.h"
 
+#include <stdarg.h>
 #include <rpm/header.h>
 #include <rpm/rpmtag.h>
 #include <rpm/rpmstring.h>
 #include <rpm/rpmpgp.h>
+#include "lib/misc.h"		/* format function protos */
 
 #include "debug.h"
 
@@ -15,41 +17,30 @@
 #define PARSER_IN_ARRAY 1
 #define PARSER_IN_EXPR  2
 
-/** \ingroup header
- * HEADER_EXT_FORMAT format function prototype.
- * This will only ever be passed RPM_INT32_TYPE or RPM_STRING_TYPE to
- * help keep things simple.
- *
- * @param td		tag data container
- * @param formatPrefix
- * @return		formatted string
- */
-typedef char * (*headerTagFormatFunction) (rpmtd td, char * formatPrefix);
-
-extern void *rpmHeaderFormatFuncByName(const char *fmt);
-
 /** \ingroup header
  */
 typedef struct sprintfTag_s * sprintfTag;
 struct sprintfTag_s {
-    headerTagFormatFunction fmt;
-    rpmTag tag;
+    headerFmt fmt;
+    rpmTagVal tag;
     int justOne;
     char * format;
-    char * type;
+    const char * type;
 };
 
+typedef enum {
+    PTOK_NONE = 0,
+    PTOK_TAG,
+    PTOK_ARRAY,
+    PTOK_STRING,
+    PTOK_COND
+} ptokType;
+
 /** \ingroup header
  */
 typedef struct sprintfToken_s * sprintfToken;
 struct sprintfToken_s {
-    enum {
-	PTOK_NONE = 0,
-	PTOK_TAG,
-	PTOK_ARRAY,
-	PTOK_STRING,
-	PTOK_COND
-    } type;
+    ptokType type;
     union {
 	struct sprintfTag_s tag;	/*!< PTOK_TAG */
 	struct {
@@ -71,13 +62,22 @@ struct sprintfToken_s {
     } u;
 };
 
+#define HASHTYPE tagCache
+#define HTKEYTYPE rpmTagVal
+#define HTDATATYPE rpmtd
+#include "lib/rpmhash.H"
+#include "lib/rpmhash.C"
+#undef HASHTYPE
+#undef HTKEYTYPE
+#undef HTDATATYPE
+
 /**
  */
 typedef struct headerSprintfArgs_s {
     Header h;
     char * fmt;
     const char * errmsg;
-    rpmtd *cache;
+    tagCache cache;
     sprintfToken format;
     HeaderIterator hi;
     char * val;
@@ -85,6 +85,7 @@ typedef struct headerSprintfArgs_s {
     size_t alloced;
     int numTokens;
     int i;
+    headerGetFlags hgflags;
 } * headerSprintfArgs;
 
 
@@ -137,7 +138,7 @@ freeFormat( sprintfToken format, int num)
 	    break;
 	}
     }
-    format = _free(format);
+    free(format);
     return NULL;
 }
 
@@ -157,6 +158,8 @@ static void hsaInit(headerSprintfArgs hsa)
     hsa->i = 0;
     if (tag != NULL && tag->tag == -2)
 	hsa->hi = headerInitIterator(hsa->h);
+    /* Normally with bells and whistles enabled, but raw dump on iteration. */
+    hsa->hgflags = (hsa->hi == NULL) ? HEADERGET_EXT : HEADERGET_RAW;
 }
 
 /**
@@ -179,13 +182,9 @@ static sprintfToken hsaNext(headerSprintfArgs hsa)
 	if (hsa->hi == NULL) {
 	    hsa->i++;
 	} else {
-	    struct rpmtd_s td;
-
-	    /* hmm, cache the data from here too? */
-	    if (!headerNext(hsa->hi, &td))
+	    tag->tag = headerNextTag(hsa->hi);
+	    if (tag->tag == RPMTAG_NOT_FOUND)
 		fmt = NULL;
-	    tag->tag = td.tag;
-	    rpmtdFreeData(&td);
 	}
     }
 
@@ -219,6 +218,25 @@ static char * hsaReserve(headerSprintfArgs hsa, size_t need)
     return hsa->val + hsa->vallen;
 }
 
+RPM_GNUC_PRINTF(2, 3)
+static void hsaError(headerSprintfArgs hsa, const char *fmt, ...)
+{
+    /* Use thread local buffer as headerFormat() errmsg arg is const */
+    static __thread char *errbuf = NULL;
+
+    if (fmt == NULL) {
+	hsa->errmsg = NULL;
+    } else {
+	va_list ap;
+
+	free(errbuf);
+	va_start(ap, fmt);
+	rvasprintf(&errbuf, fmt, ap);
+	va_end(ap);
+	hsa->errmsg = errbuf;
+    }
+}
+
 /**
  * Search tags for a name.
  * @param hsa		headerSprintf args
@@ -233,28 +251,22 @@ static int findTag(headerSprintfArgs hsa, sprintfToken token, const char * name)
 	? &token->u.cond.tag : &token->u.tag);
 
     stag->fmt = NULL;
-    stag->tag = -1;
+    stag->tag = RPMTAG_NOT_FOUND;
 
-    if (rstreq(tagname, "*")) {
-	stag->tag = -2;
-	goto bingo;
-    }
+    if (!rstreq(tagname, "*")) {
+	if (rstreqn("RPMTAG_", tagname, sizeof("RPMTAG_")-1)) {
+	    tagname += sizeof("RPMTAG");
+	}
 
-    if (rstreqn("RPMTAG_", tagname, sizeof("RPMTAG_")-1)) {
-	tagname += sizeof("RPMTAG");
-    }
+	/* Search tag names. */
+	stag->tag = rpmTagGetValue(tagname);
+	if (stag->tag == RPMTAG_NOT_FOUND) return 1;
 
-    /* Search tag names. */
-    stag->tag = rpmTagGetValue(tagname);
-    if (stag->tag != RPMTAG_NOT_FOUND)
-	goto bingo;
+    } else stag->tag = -2;
 
-    return 1;
-
-bingo:
     /* Search extensions for specific format. */
     if (stag->type != NULL)
-	stag->fmt = rpmHeaderFormatFuncByName(stag->type);
+	stag->fmt = rpmHeaderFormatByName(stag->type);
 
     return stag->fmt ? 0 : 1;
 }
@@ -275,9 +287,9 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
  * Parse a headerSprintf term.
  * @param hsa		headerSprintf args
  * @param str
- * @retval *formatPtr
- * @retval *numTokensPtr
- * @retval *endPtr
+ * @param[out] *formatPtr
+ * @param[out] *numTokensPtr
+ * @param[out] *endPtr
  * @param state
  * @return		0 on success
  */
@@ -339,9 +351,15 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    token->u.tag.justOne = 0;
 
 	    chptr = start;
-	    while (*chptr && *chptr != '{' && *chptr != '%') chptr++;
+	    while (*chptr && *chptr != '{' && *chptr != '%') {
+		if (!risdigit(*chptr) && *chptr != '-') {
+		    hsaError(hsa, _("invalid field width"));
+		    goto errxit;
+		}
+		chptr++;
+	    }
 	    if (!*chptr || *chptr == '%') {
-		hsa->errmsg = _("missing { after %");
+		hsaError(hsa, _("missing { after %%"));
 		goto errxit;
 	    }
 
@@ -363,7 +381,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    dst = next = start;
 	    while (*next && *next != '}') next++;
 	    if (!*next) {
-		hsa->errmsg = _("missing } after %{");
+		hsaError(hsa, _("missing } after %%{"));
 		goto errxit;
 	    }
 	    *next++ = '\0';
@@ -374,7 +392,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    if (*chptr != '\0') {
 		*chptr++ = '\0';
 		if (!*chptr) {
-		    hsa->errmsg = _("empty tag format");
+		    hsaError(hsa, _("empty tag format"));
 		    goto errxit;
 		}
 		token->u.tag.type = chptr;
@@ -385,14 +403,14 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    }
 	    
 	    if (!*start) {
-		hsa->errmsg = _("empty tag name");
+		hsaError(hsa, _("empty tag name"));
 		goto errxit;
 	    }
 
 	    token->type = PTOK_TAG;
 
 	    if (findTag(hsa, token, start)) {
-		hsa->errmsg = _("unknown tag");
+		hsaError(hsa, _("unknown tag: \"%s\""), start);
 		goto errxit;
 	    }
 
@@ -412,7 +430,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    }
 
 	    if (!start) {
-		hsa->errmsg = _("] expected at end of array");
+		hsaError(hsa, _("] expected at end of array"));
 		goto errxit;
 	    }
 
@@ -424,7 +442,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 
 	case ']':
 	    if (state != PARSER_IN_ARRAY) {
-		hsa->errmsg = _("unexpected ]");
+		hsaError(hsa, _("unexpected ]"));
 		goto errxit;
 	    }
 	    *start++ = '\0';
@@ -434,7 +452,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 
 	case '}':
 	    if (state != PARSER_IN_EXPR) {
-		hsa->errmsg = _("unexpected }");
+		hsaError(hsa, _("unexpected }"));
 		goto errxit;
 	    }
 	    *start++ = '\0';
@@ -451,6 +469,10 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 
 	    if (*start == '\\') {
 		start++;
+		if (*start == '\0') {
+		    hsaError(hsa, _("escaped char expected after \\"));
+		    goto errxit;
+		}
 		*dst++ = escapedChar(*start++);
 	    } else {
 		*dst++ = *start++;
@@ -485,19 +507,19 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
     char * chptr;
     char * end;
 
-    hsa->errmsg = NULL;
+    hsaError(hsa, NULL);
     chptr = str;
     while (*chptr && *chptr != '?') chptr++;
 
     if (*chptr != '?') {
-	hsa->errmsg = _("? expected in expression");
+	hsaError(hsa, _("? expected in expression"));
 	return 1;
     }
 
     *chptr++ = '\0';;
 
     if (*chptr != '{') {
-	hsa->errmsg = _("{ expected after ? in expression");
+	hsaError(hsa, _("{ expected after ? in expression"));
 	return 1;
     }
 
@@ -509,7 +531,7 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 
     /* XXX fix segfault on "rpm -q rpm --qf='%|NAME?{%}:{NAME}|\n'"*/
     if (!(end && *end)) {
-	hsa->errmsg = _("} expected in expression");
+	hsaError(hsa, _("} expected in expression"));
 	token->u.cond.ifFormat =
 		freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	return 1;
@@ -517,7 +539,7 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 
     chptr = end;
     if (*chptr != ':' && *chptr != '|') {
-	hsa->errmsg = _(": expected following ? subexpression");
+	hsaError(hsa, _(": expected following ? subexpression"));
 	token->u.cond.ifFormat =
 		freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	return 1;
@@ -535,7 +557,7 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 	chptr++;
 
 	if (*chptr != '{') {
-	    hsa->errmsg = _("{ expected after : in expression");
+	    hsaError(hsa, _("{ expected after : in expression"));
 	    token->u.cond.ifFormat =
 		freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	    return 1;
@@ -549,7 +571,7 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 
 	/* XXX fix segfault on "rpm -q rpm --qf='%|NAME?{a}:{%}|{NAME}\n'" */
 	if (!(end && *end)) {
-	    hsa->errmsg = _("} expected in expression");
+	    hsaError(hsa, _("} expected in expression"));
 	    token->u.cond.ifFormat =
 		freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	    return 1;
@@ -557,7 +579,7 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 
 	chptr = end;
 	if (*chptr != '|') {
-	    hsa->errmsg = _("| expected at end of expression");
+	    hsaError(hsa, _("| expected at end of expression"));
 	    token->u.cond.ifFormat =
 		freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	    token->u.cond.elseFormat =
@@ -577,36 +599,32 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
     return 0;
 }
 
-static rpmtd getCached(rpmtd *cache, rpmTag tag)
+static rpmtd getCached(tagCache cache, rpmTagVal tag)
 {
-    rpmtd td = NULL;
-
-    if (tag >= HEADER_IMAGE && tag < RPMTAG_FIRSTFREE_TAG && cache[tag]) {
-	td = cache[tag];
-    }
-    return td;
+    rpmtd *res = NULL;
+    return tagCacheGetEntry(cache, tag, &res, NULL, NULL) ? res[0] : NULL;
 }
 
 /**
  * Do headerGet() just once for given tag, cache results.
  * @param hsa		headerSprintf args
  * @param tag
- * @retval *typeptr
- * @retval *data
- * @retval *countptr
+ * @param[out] *typeptr
+ * @param[out] *data
+ * @param[out] *countptr
  * @return		1 on success, 0 on failure
  */
-static rpmtd getData(headerSprintfArgs hsa, rpmTag tag)
+static rpmtd getData(headerSprintfArgs hsa, rpmTagVal tag)
 {
     rpmtd td = NULL;
 
     if (!(td = getCached(hsa->cache, tag))) {
 	td = rpmtdNew();
-	if (!headerGet(hsa->h, tag, td, HEADERGET_EXT)) {
+	if (!headerGet(hsa->h, tag, td, hsa->hgflags)) {
 	    rpmtdFree(td);
 	    return NULL;
 	}
-	hsa->cache[tag] = td;
+	tagCacheAddEntry(hsa->cache, tag, td);
     }
 
     return td;
@@ -624,19 +642,27 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
     char * val = NULL;
     size_t need = 0;
     char * t, * te;
-    char buf[20];
     rpmtd td;
 
-    memset(buf, 0, sizeof(buf));
-    if ((td = getData(hsa, tag->tag))) {
+    if ((td = getData(hsa, tag->tag)) && td->count > element) {
 	td->ix = element; /* Ick, use iterators instead */
-	stpcpy(stpcpy(buf, "%"), tag->format);
-	val = tag->fmt(td, buf);
+	val = rpmHeaderFormatCall(tag->fmt, td);
     } else {
-	stpcpy(buf, "%s");
 	val = xstrdup("(none)");
     }
 
+    /* Handle field width + justification formatting if specified */
+    if (tag->format && *tag->format) {
+	char *tval = NULL;
+	/* user string + extra for '%', format char and trailing '\0' */
+	char fmtbuf[strlen(tag->format) + 3];
+
+	sprintf(fmtbuf, "%%%ss", tag->format);
+	rasprintf(&tval, fmtbuf, val);
+	free(val);
+	val = tval;
+    }
+	
     need = strlen(val);
 
     if (val && need > 0) {
@@ -726,16 +752,10 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 	    found = 1;
 	    count = rpmtdCount(td);
 
-	    if (numElements > 1 && count != numElements)
-	    switch (td->type) {
-	    default:
-		hsa->errmsg =
-			_("array iterator used with different sized arrays");
+	    if (numElements > 0 && count != numElements) {
+		hsaError(hsa,
+			_("array iterator used with different sized arrays"));
 		return NULL;
-		break;
-	    case RPM_BIN_TYPE:
-	    case RPM_STRING_TYPE:
-		break;
 	    }
 	    if (count > numElements)
 		numElements = count;
@@ -789,31 +809,14 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
     return (hsa->val + hsa->vallen);
 }
 
-/**
- * Create tag data cache.
- * This allocates much more space than necessary but playing it
- * simple and stupid for now.
- */
-static rpmtd *cacheCreate(void)
+static int tagCmp(rpmTagVal a, rpmTagVal b)
 {
-    rpmtd *cache = xcalloc(RPMTAG_FIRSTFREE_TAG, sizeof(*cache));
-    return cache;
+    return (a != b);
 }
 
-/**
- * Free tag data cache contents and destroy cache.
- */
-static void *cacheFree(rpmtd *cache)
+static unsigned int tagId(rpmTagVal tag)
 {
-    rpmtd *td = cache;
-    for (int i = 0; i < RPMTAG_FIRSTFREE_TAG; i++, td++) {
-	if (*td) {
-	    rpmtdFreeData(*td);
-	    rpmtdFree(*td);
-	}
-    }
-    free(cache);
-    return NULL;
+    return tag;
 }
 
 char * headerFormat(Header h, const char * fmt, errmsg_t * errmsg) 
@@ -833,7 +836,7 @@ char * headerFormat(Header h, const char * fmt, errmsg_t * errmsg)
     if (parseFormat(&hsa, hsa.fmt, &hsa.format, &hsa.numTokens, NULL, PARSER_BEGIN))
 	goto exit;
 
-    hsa.cache = cacheCreate();
+    hsa.cache = tagCacheCreate(128, tagId, tagCmp, NULL, rpmtdFree);
     hsa.val = xstrdup("");
 
     tag =
@@ -871,7 +874,7 @@ char * headerFormat(Header h, const char * fmt, errmsg_t * errmsg)
     if (hsa.val != NULL && hsa.vallen < hsa.alloced)
 	hsa.val = xrealloc(hsa.val, hsa.vallen+1);	
 
-    hsa.cache = cacheFree(hsa.cache);
+    hsa.cache = tagCacheFree(hsa.cache);
     hsa.format = freeFormat(hsa.format, hsa.numTokens);
 
 exit:
