@@ -67,12 +67,13 @@ static void rpmTSIFree(tsortInfo tsi)
 
 static inline int addSingleRelation(rpmte p,
 				    rpmte q,
-				    rpmsenseFlags dsflags,
-				    int reversed)
+				    rpmds dep)
 {
     struct tsortInfo_s *tsi_p, *tsi_q;
     relation rel;
     rpmElementType teType = rpmteType(p);
+    rpmsenseFlags dsflags = rpmdsFlags(dep);
+    int reversed = rpmdsIsReverse(dep);
     rpmsenseFlags flags;
 
     /* Avoid deps outside this transaction and self dependencies */
@@ -165,53 +166,49 @@ static inline int addSingleRelation(rpmte p,
  * @param ts		transaction set
  * @param al		packages list
  * @param p		predecessor (i.e. package that "Requires: q")
- * @param requires	relation
+ * @param dep		dependency relation
  * @return		0 always
  */
 static inline int addRelation(rpmts ts,
 			      rpmal al,
 			      rpmte p,
-			      rpmds requires,
-			      int reversed)
+			      rpmds dep)
 {
     rpmte q;
-    rpmsenseFlags dsflags;
-
-    dsflags = rpmdsFlags(requires);
 
     /* Avoid dependendencies which are not relevant for ordering */
-    if (dsflags & (RPMSENSE_RPMLIB|RPMSENSE_CONFIG|RPMSENSE_PRETRANS|RPMSENSE_POSTTRANS))
+    if (isUnorderedReq(rpmdsFlags(dep)))
 	return 0;
 
-    if (rpmdsIsRich(requires)) {
+    if (rpmdsIsRich(dep)) {
 	rpmds ds1, ds2;
 	rpmrichOp op;
-	if (rpmdsParseRichDep(requires, &ds1, &ds2, &op, NULL) == RPMRC_OK) {
+	if (rpmdsParseRichDep(dep, &ds1, &ds2, &op, NULL) == RPMRC_OK) {
 	    if (op != RPMRICHOP_ELSE)
-		addRelation(ts, al, p, ds1, reversed);
+		addRelation(ts, al, p, ds1);
 	    if (op == RPMRICHOP_IF || op == RPMRICHOP_UNLESS) {
 	      rpmds ds21, ds22;
 	      rpmrichOp op2;
-	      if (rpmdsParseRichDep(requires, &ds21, &ds22, &op2, NULL) == RPMRC_OK && op2 == RPMRICHOP_ELSE) {
-		  addRelation(ts, al, p, ds22, reversed);
+	      if (rpmdsParseRichDep(dep, &ds21, &ds22, &op2, NULL) == RPMRC_OK && op2 == RPMRICHOP_ELSE) {
+		  addRelation(ts, al, p, ds22);
 	      }
 	      ds21 = rpmdsFree(ds21);
 	      ds22 = rpmdsFree(ds22);
 	    }
 	    if (op == RPMRICHOP_AND || op == RPMRICHOP_OR)
-		addRelation(ts, al, p, ds2, reversed);
+		addRelation(ts, al, p, ds2);
 	    ds1 = rpmdsFree(ds1);
 	    ds2 = rpmdsFree(ds2);
 	}
 	return 0;
     }
-    q = rpmalSatisfiesDepend(al, p, requires);
+    q = rpmalSatisfiesDepend(al, p, dep);
 
     /* Avoid deps outside this transaction and self dependencies */
     if (q == NULL || q == p)
 	return 0;
 
-    addSingleRelation(p, q, dsflags, reversed);
+    addSingleRelation(p, q, dep);
 
     return 0;
 }
@@ -219,8 +216,8 @@ static inline int addRelation(rpmts ts,
 /**
  * Add element to list sorting by tsi_qcnt.
  * @param p		new element
- * @retval qp		address of first element
- * @retval rp		address of last element
+ * @param[out] qp		address of first element
+ * @param[out] rp		address of last element
  * @param prefcolor
  */
 static void addQ(tsortInfo p, tsortInfo * qp, tsortInfo * rp,
@@ -588,41 +585,20 @@ int rpmtsOrder(rpmts ts)
     while ((p = rpmtsiNext(pi, 0)) != NULL) {
 	rpmal al = (rpmteType(p) == TR_REMOVED) ? 
 		   erasedPackages : tsmem->addedPackages;
-	rpmds requires = rpmdsInit(rpmteDS(p, RPMTAG_REQUIRENAME));
-	rpmds recommends = rpmdsInit(rpmteDS(p, RPMTAG_RECOMMENDNAME));
-	rpmds suggests = rpmdsInit(rpmteDS(p, RPMTAG_SUGGESTNAME));
-	rpmds supplements = rpmdsInit(rpmteDS(p, RPMTAG_SUPPLEMENTNAME));
-	rpmds enhances = rpmdsInit(rpmteDS(p, RPMTAG_ENHANCENAME));
-	rpmds order = rpmdsInit(rpmteDS(p, RPMTAG_ORDERNAME));
+	rpmTag ordertags[] = {
+		RPMTAG_REQUIRENAME,
+		RPMTAG_RECOMMENDNAME,
+		RPMTAG_SUGGESTNAME,
+		RPMTAG_SUPPLEMENTNAME,
+		RPMTAG_ENHANCENAME,
+		RPMTAG_ORDERNAME,
+		0,
+	};
 
-	while (rpmdsNext(requires) >= 0) {
-	    /* Record next "q <- p" relation (i.e. "p" requires "q"). */
-	    (void) addRelation(ts, al, p, requires, 0);
-	}
-
-	while (rpmdsNext(recommends) >= 0) {
-	    /* Record next "q <- p" relation (i.e. "p" recommends "q"). */
-	    (void) addRelation(ts, al, p, recommends, 0);
-	}
-
-	while (rpmdsNext(suggests) >= 0) {
-	    /* Record next "q <- p" relation (i.e. "p" suggests "q"). */
-	    (void) addRelation(ts, al, p, suggests, 0);
-	}
-
-	while (rpmdsNext(order) >= 0) {
-	    /* Record next "q <- p" ordering request */
-	    (void) addRelation(ts, al, p, order, 0);
-	}
-
-	while (rpmdsNext(supplements) >= 0) {
-	    /* Record next "p -> q" relation (i.e. "q" supplemented by "p"). */
-	    (void) addRelation(ts, al, p, supplements, 1);
-	}
-
-	while (rpmdsNext(enhances) >= 0) {
-	    /* Record next "p <- q" relation (i.e. "q" is enhanced by  "p"). */
-	    (void) addRelation(ts, al, p, enhances, 1);
+	for (int i = 0; ordertags[i]; i++) {
+	    rpmds dep = rpmdsInit(rpmteDS(p, ordertags[i]));
+	    while (rpmdsNext(dep) >= 0)
+		addRelation(ts, al, p, dep);
 	}
     }
 
