@@ -21,6 +21,12 @@ struct rpmQVKArguments_s rpmQVKArgs;
 #define POPT_QUERYBYPKGID	-1007
 #define POPT_QUERYBYHDRID	-1008
 #define POPT_QUERYBYTID		-1010
+#define POPT_WHATRECOMMENDS	-1011
+#define POPT_WHATSUGGESTS	-1012
+#define POPT_WHATSUPPLEMENTS	-1013
+#define POPT_WHATENHANCES	-1014
+#define POPT_WHATOBSOLETES	-1015
+#define POPT_WHATCONFLICTS	-1016
 
 /* ========== Query/Verify/Signature source args */
 static void rpmQVSourceArgCallback( poptContext con,
@@ -44,7 +50,13 @@ static void rpmQVSourceArgCallback( poptContext con,
     case 'g': qva->qva_source |= RPMQV_GROUP; break;
     case 'p': qva->qva_source |= RPMQV_RPM; break;
     case POPT_WHATPROVIDES: qva->qva_source |= RPMQV_WHATPROVIDES; break;
+    case POPT_WHATOBSOLETES: qva->qva_source |= RPMQV_WHATOBSOLETES; break;
     case POPT_WHATREQUIRES: qva->qva_source |= RPMQV_WHATREQUIRES; break;
+    case POPT_WHATCONFLICTS: qva->qva_source |= RPMQV_WHATCONFLICTS; break;
+    case POPT_WHATRECOMMENDS: qva->qva_source |= RPMQV_WHATRECOMMENDS; break;
+    case POPT_WHATSUGGESTS: qva->qva_source |= RPMQV_WHATSUGGESTS; break;
+    case POPT_WHATSUPPLEMENTS: qva->qva_source |= RPMQV_WHATSUPPLEMENTS; break;
+    case POPT_WHATENHANCES: qva->qva_source |= RPMQV_WHATENHANCES; break;
     case POPT_TRIGGEREDBY: qva->qva_source |= RPMQV_TRIGGEREDBY; break;
     case POPT_QUERYBYPKGID: qva->qva_source |= RPMQV_PKGID; break;
     case POPT_QUERYBYHDRID: qva->qva_source |= RPMQV_HDRID; break;
@@ -89,10 +101,22 @@ struct poptOption rpmQVSourcePoptTable[] = {
 	N_("query the package(s) triggered by the package"), "PACKAGE" },
  { "verify", 'V', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'V',
 	N_("rpm verify mode"), NULL },
+ { "whatconflicts", '\0', 0, 0, POPT_WHATCONFLICTS, 
+	N_("query/verify the package(s) which require a dependency"), "CAPABILITY" },
  { "whatrequires", '\0', 0, 0, POPT_WHATREQUIRES, 
 	N_("query/verify the package(s) which require a dependency"), "CAPABILITY" },
+ { "whatobsoletes", '\0', 0, 0, POPT_WHATOBSOLETES,
+	N_("query/verify the package(s) which obsolete a dependency"), "CAPABILITY" },
  { "whatprovides", '\0', 0, 0, POPT_WHATPROVIDES, 
 	N_("query/verify the package(s) which provide a dependency"), "CAPABILITY" },
+ { "whatrecommends", '\0', 0, 0, POPT_WHATRECOMMENDS,
+	N_("query/verify the package(s) which recommends a dependency"), "CAPABILITY" },
+ { "whatsuggests", '\0', 0, 0, POPT_WHATSUGGESTS,
+	N_("query/verify the package(s) which suggests a dependency"), "CAPABILITY" },
+ { "whatsupplements", '\0', 0, 0, POPT_WHATSUPPLEMENTS,
+	N_("query/verify the package(s) which supplements a dependency"), "CAPABILITY" },
+ { "whatenhances", '\0', 0, 0, POPT_WHATENHANCES,
+	N_("query/verify the package(s) which enhances a dependency"), "CAPABILITY" },
 
  { "noglob", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN, &giFlags, RPMGI_NOGLOB,
 	N_("do not glob arguments"), NULL},
@@ -112,8 +136,6 @@ static void queryArgCallback(poptContext con,
     QVA_t qva = &rpmQVKArgs;
 
     switch (opt->val) {
-    case 'c': qva->qva_flags |= QUERY_FOR_CONFIG | QUERY_FOR_LIST; break;
-    case 'd': qva->qva_flags |= QUERY_FOR_DOCS | QUERY_FOR_LIST; break;
     case 'l': qva->qva_flags |= QUERY_FOR_LIST; break;
     case 's': qva->qva_flags |= QUERY_FOR_STATE | QUERY_FOR_LIST;
 	break;
@@ -136,11 +158,15 @@ static void queryArgCallback(poptContext con,
 	break;
 
     case RPMCLI_POPT_NOFILEDIGEST:
-	qva->qva_flags |= VERIFY_FILEDIGEST;
+	qva->qva_ofvattr |= RPMVERIFY_FILEDIGEST;
 	break;
 
     case RPMCLI_POPT_NOCONTEXTS:
-	qva->qva_flags |= VERIFY_CONTEXTS;
+	qva->qva_ofvattr |= RPMVERIFY_CONTEXTS;
+	break;
+
+    case RPMCLI_POPT_NOCAPS:
+	qva->qva_ofvattr |= RPMVERIFY_CAPS;
 	break;
 
 #ifdef	NOTYET
@@ -160,29 +186,45 @@ static void queryArgCallback(poptContext con,
     }
 }
 
+ /* Duplicate file attr flags from packages into command line options. */
+struct poptOption rpmQVFilePoptTable[] = {
+ { "configfiles", 'c', POPT_BIT_SET,
+	&rpmQVKArgs.qva_incattr, RPMFILE_CONFIG,
+	N_("only include configuration files"), NULL },
+ { "docfiles", 'd', POPT_BIT_SET,
+	&rpmQVKArgs.qva_incattr, RPMFILE_DOC,
+	N_("only include documentation files"), NULL },
+ { "licensefiles", 'L', POPT_BIT_SET,
+	&rpmQVKArgs.qva_incattr, RPMFILE_LICENSE,
+	N_("only include license files"), NULL },
+ { "artifactfiles", 'A', POPT_BIT_SET,
+	&rpmQVKArgs.qva_incattr, RPMFILE_ARTIFACT,
+	N_("only include artifact files"), NULL },
+ { "noghost", '\0', POPT_BIT_SET,
+	&rpmQVKArgs.qva_excattr, RPMFILE_GHOST,
+        N_("exclude %%ghost files"), NULL },
+ { "noconfig", '\0', POPT_BIT_SET,
+	&rpmQVKArgs.qva_excattr, RPMFILE_CONFIG,
+        N_("exclude %%config files"), NULL },
+ { "noartifact", '\0', POPT_BIT_SET,
+	&rpmQVKArgs.qva_excattr, RPMFILE_ARTIFACT,
+        N_("exclude %%artifact files"), NULL },
+
+   POPT_TABLEEND
+};
+
 /**
  * Query mode options.
  */
 struct poptOption rpmQueryPoptTable[] = {
-/* FIX: cast? */
- { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE, 
+ { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE,
 	queryArgCallback, 0, NULL, NULL },
- { "configfiles", 'c', 0, 0, 'c',
-	N_("list all configuration files"), NULL },
- { "docfiles", 'd', 0, 0, 'd',
-	N_("list all documentation files"), NULL },
  { "dump", '\0', 0, 0, POPT_DUMP,
 	N_("dump basic file information"), NULL },
  { NULL, 'i', POPT_ARGFLAG_DOC_HIDDEN, 0, 'i',
 	NULL, NULL },
  { "list", 'l', 0, 0, 'l',
 	N_("list files in package"), NULL },
-
- /* Duplicate file attr flags from packages into command line options. */
- { "noghost", '\0', POPT_BIT_CLR|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_fflags, RPMFILE_GHOST,
-        N_("skip %%ghost files"), NULL },
-
  { "qf", '\0', POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, 0, 
 	POPT_QUERYFORMAT, NULL, NULL },
  { "queryformat", '\0', POPT_ARG_STRING, 0, POPT_QUERYFORMAT,
@@ -205,32 +247,31 @@ struct poptOption rpmVerifyPoptTable[] = {
  { "nomd5", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOFILEDIGEST,
 	N_("don't verify digest of files"), NULL },
  { "nosize", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_SIZE,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_FILESIZE,
         N_("don't verify size of files"), NULL },
  { "nolinkto", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_LINKTO,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_LINKTO,
         N_("don't verify symlink path of files"), NULL },
  { "nouser", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_USER,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_USER,
         N_("don't verify owner of files"), NULL },
  { "nogroup", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_GROUP,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_GROUP,
         N_("don't verify group of files"), NULL },
  { "nomtime", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_MTIME,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_MTIME,
         N_("don't verify modification time of files"), NULL },
  { "nomode", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_MODE,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_MODE,
         N_("don't verify mode of files"), NULL },
  { "nordev", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_RDEV,
+	&rpmQVKArgs.qva_ofvattr, RPMVERIFY_RDEV,
         N_("don't verify mode of files"), NULL },
- { "nocaps", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_CAPS,
-	N_("don't verify capabilities of files"), NULL },
 
  { "nocontexts", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOCONTEXTS,
 	N_("don't verify file security contexts"), NULL },
+ { "nocaps", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOCAPS,
+	N_("don't verify capabilities of files"), NULL },
  { "nofiles", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_FILES,
 	N_("don't verify files in package"), NULL},
  { "nodeps", '\0', 0, NULL, RPMCLI_POPT_NODEPS,
