@@ -16,17 +16,13 @@ extern "C" {
 typedef struct rpmal_s * rpmal;
 
 /**
- * Initialize available packckages, items, and directory list.
- * @param pool		shared string pool with base, dir and dependency names
+ * Initialize available packages, items, and directory list.
+ * @param ts		transaction set
  * @param delta		no. of entries to add on each realloc
- * @param tsflags	transaction control flags
- * @param tscolor	transaction color bits
- * @param prefcolor	preferred color
  * @return al		new available list
  */
 RPM_GNUC_INTERNAL
-rpmal rpmalCreate(rpmstrPool pool, int delta, rpmtransFlags tsflags,
-		  rpm_color_t tscolor, rpm_color_t prefcolor);
+rpmal rpmalCreate(rpmts ts, int delta);
 
 /**
  * Free available packages, items, and directory members.
