@@ -1,25 +1,18 @@
 #ifndef _RPMTAG_H
 #define _RPMTAG_H
 
+/** \ingroup rpmtag
+ *  \file lib/rpmtag.h
+ *
+ * Accessing RPM tags: values, types, ...
+ */
+
 #include <rpm/rpmtypes.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/**
- * Pseudo-tags used by the rpmdb and rpmgi iterator API's.
- */
-#define	RPMDBI_PACKAGES		0	/* Installed package headers. */
-#define	RPMDBI_DEPENDS		1	/* Dependency resolution cache. */
-#define	RPMDBI_LABEL		2	/* Fingerprint search marker. */
-#define	RPMDBI_ADDED		3	/* Added package headers. */
-#define	RPMDBI_REMOVED		4	/* Removed package headers. */
-#define	RPMDBI_AVAILABLE	5	/* Available package headers. */
-#define	RPMDBI_HDLIST		6	/* (rpmgi) Header list. */
-#define	RPMDBI_ARGLIST		7	/* (rpmgi) Argument list. */
-#define	RPMDBI_FTSWALK		8	/* (rpmgi) File tree  walk. */
-
 /**
  * Header private tags.
  * @note General use tags should start at 1000 (RPM's tag space starts there).
@@ -35,9 +28,11 @@ extern "C" {
 /** \ingroup rpmtag
  * Tags identify data in package headers.
  * @note tags should not have value 0!
+ * @note all new tags should be added above 5000
  */
 /** @todo: Somehow supply type **/
 typedef enum rpmTag_e {
+    RPMTAG_NOT_FOUND		= -1,			/*!< Unknown tag */
 
     RPMTAG_HEADERIMAGE		= HEADER_IMAGE,		/*!< Current image. */
     RPMTAG_HEADERSIGNATURES	= HEADER_SIGNATURES,	/*!< Signatures. */
@@ -68,6 +63,12 @@ typedef enum rpmTag_e {
 #define	RPMTAG_HDRID	RPMTAG_SHA1HEADER	/* s */
     RPMTAG_LONGSIGSIZE		= RPMTAG_SIG_BASE+14,	/* l */
     RPMTAG_LONGARCHIVESIZE	= RPMTAG_SIG_BASE+15,	/* l */
+    /* RPMTAG_SIG_BASE+16 reserved */
+    RPMTAG_SHA256HEADER		= RPMTAG_SIG_BASE+17,	/* s */
+    /* RPMTAG_SIG_BASE+18 reserved for RPMSIGTAG_FILESIGNATURES */
+    /* RPMTAG_SIG_BASE+19 reserved for RPMSIGTAG_FILESIGNATURELENGTH */
+    RPMTAG_VERITYSIGNATURES	= RPMTAG_SIG_BASE+20,	/* s[] */
+    RPMTAG_VERITYSIGNATUREALGO	= RPMTAG_SIG_BASE+21,	/* i */
 
     RPMTAG_NAME  		= 1000,	/* s */
 #define	RPMTAG_N	RPMTAG_NAME	/* s */
@@ -104,8 +105,8 @@ typedef enum rpmTag_e {
     RPMTAG_FILESIZES		= 1028,	/* i[] */
     RPMTAG_FILESTATES		= 1029, /* c[] */
     RPMTAG_FILEMODES		= 1030,	/* h[] */
-    RPMTAG_FILEUIDS		= 1031, /* i[] internal */
-    RPMTAG_FILEGIDS		= 1032, /* i[] internal */
+    RPMTAG_FILEUIDS		= 1031, /* i[] internal - obsolete */
+    RPMTAG_FILEGIDS		= 1032, /* i[] internal - obsolete */
     RPMTAG_FILERDEVS		= 1033,	/* h[] */
     RPMTAG_FILEMTIMES		= 1034, /* i[] */
     RPMTAG_FILEDIGESTS		= 1035,	/* s[] */
@@ -128,15 +129,15 @@ typedef enum rpmTag_e {
     RPMTAG_REQUIRENAME		= 1049,	/* s[] */
 #define	RPMTAG_REQUIRES RPMTAG_REQUIRENAME	/* s[] */
     RPMTAG_REQUIREVERSION	= 1050,	/* s[] */
-    RPMTAG_NOSOURCE		= 1051, /* i */
-    RPMTAG_NOPATCH		= 1052, /* i */
+    RPMTAG_NOSOURCE		= 1051, /* i[] */
+    RPMTAG_NOPATCH		= 1052, /* i[] */
     RPMTAG_CONFLICTFLAGS	= 1053, /* i[] */
     RPMTAG_CONFLICTNAME		= 1054,	/* s[] */
 #define	RPMTAG_CONFLICTS RPMTAG_CONFLICTNAME	/* s[] */
 #define	RPMTAG_C	RPMTAG_CONFLICTNAME	/* s[] */
     RPMTAG_CONFLICTVERSION	= 1055,	/* s[] */
     RPMTAG_DEFAULTPREFIX	= 1056, /* s internal - deprecated */
-    RPMTAG_BUILDROOT		= 1057, /* s internal */
+    RPMTAG_BUILDROOT		= 1057, /* s internal - obsolete */
     RPMTAG_INSTALLPREFIX	= 1058, /* s internal - deprecated */
     RPMTAG_EXCLUDEARCH		= 1059, /* s[] */
     RPMTAG_EXCLUDEOS		= 1060, /* s[] */
@@ -155,15 +156,15 @@ typedef enum rpmTag_e {
     RPMTAG_CHANGELOGTEXT	= 1082,	/* s[] */
     RPMTAG_BROKENMD5		= 1083, /* internal - obsolete */
     RPMTAG_PREREQ		= 1084, /* internal */
-    RPMTAG_PREINPROG		= 1085,	/* s */
-    RPMTAG_POSTINPROG		= 1086,	/* s */
-    RPMTAG_PREUNPROG		= 1087,	/* s */
-    RPMTAG_POSTUNPROG		= 1088,	/* s */
+    RPMTAG_PREINPROG		= 1085,	/* s[] */
+    RPMTAG_POSTINPROG		= 1086,	/* s[] */
+    RPMTAG_PREUNPROG		= 1087,	/* s[] */
+    RPMTAG_POSTUNPROG		= 1088,	/* s[] */
     RPMTAG_BUILDARCHS		= 1089, /* s[] */
     RPMTAG_OBSOLETENAME		= 1090,	/* s[] */
 #define	RPMTAG_OBSOLETES RPMTAG_OBSOLETENAME	/* s[] */
 #define	RPMTAG_O	RPMTAG_OBSOLETENAME	/* s[] */
-    RPMTAG_VERIFYSCRIPTPROG	= 1091,	/* s */
+    RPMTAG_VERIFYSCRIPTPROG	= 1091,	/* s[] */
     RPMTAG_TRIGGERSCRIPTPROG	= 1092,	/* s[] */
     RPMTAG_DOCDIR		= 1093, /* internal */
     RPMTAG_COOKIE		= 1094,	/* s */
@@ -177,8 +178,8 @@ typedef enum rpmTag_e {
     RPMTAG_TRIGGERPOSTUN	= 1102, /* internal */
     RPMTAG_AUTOREQ		= 1103, /* internal */
     RPMTAG_AUTOPROV		= 1104, /* internal */
-    RPMTAG_CAPABILITY		= 1105, /* i legacy - obsolete */
-    RPMTAG_SOURCEPACKAGE	= 1106, /* i legacy - obsolete */
+    RPMTAG_CAPABILITY		= 1105, /* i internal - obsolete */
+    RPMTAG_SOURCEPACKAGE	= 1106, /* i */
     RPMTAG_OLDORIGFILENAMES	= 1107, /* internal - obsolete */
     RPMTAG_BUILDPREREQ		= 1108, /* internal */
     RPMTAG_BUILDREQUIRES	= 1109, /* internal */
@@ -203,7 +204,7 @@ typedef enum rpmTag_e {
     RPMTAG_INSTALLTID		= 1128,	/* i */
     RPMTAG_REMOVETID		= 1129,	/* i */
     RPMTAG_SHA1RHN		= 1130, /* internal - obsolete */
-    RPMTAG_RHNPLATFORM		= 1131,	/* s deprecated */
+    RPMTAG_RHNPLATFORM		= 1131,	/* s internal - obsolete */
     RPMTAG_PLATFORM		= 1132,	/* s */
     RPMTAG_PATCHESNAME		= 1133, /* s[] deprecated placeholder (SuSE) */
     RPMTAG_PATCHESFLAGS		= 1134, /* i[] deprecated placeholder (SuSE) */
@@ -225,17 +226,17 @@ typedef enum rpmTag_e {
     RPMTAG_POLICIES		= 1150,	/* s[] selinux *.te policy file. */
     RPMTAG_PRETRANS		= 1151,	/* s */
     RPMTAG_POSTTRANS		= 1152,	/* s */
-    RPMTAG_PRETRANSPROG		= 1153,	/* s */
-    RPMTAG_POSTTRANSPROG	= 1154,	/* s */
+    RPMTAG_PRETRANSPROG		= 1153,	/* s[] */
+    RPMTAG_POSTTRANSPROG	= 1154,	/* s[] */
     RPMTAG_DISTTAG		= 1155,	/* s */
-    RPMTAG_SUGGESTSNAME		= 1156,	/* s[] extension (unimplemented) */
-#define	RPMTAG_SUGGESTS RPMTAG_SUGGESTSNAME	/* s[] (unimplemented) */
-    RPMTAG_SUGGESTSVERSION	= 1157,	/* s[] extension (unimplemented) */
-    RPMTAG_SUGGESTSFLAGS	= 1158,	/* i[] extension (unimplemented) */
-    RPMTAG_ENHANCESNAME		= 1159,	/* s[] extension placeholder (unimplemented) */
-#define	RPMTAG_ENHANCES RPMTAG_ENHANCESNAME	/* s[] (unimplemented) */
-    RPMTAG_ENHANCESVERSION	= 1160,	/* s[] extension placeholder (unimplemented) */
-    RPMTAG_ENHANCESFLAGS	= 1161,	/* i[] extension placeholder (unimplemented) */
+    RPMTAG_OLDSUGGESTSNAME	= 1156, /* s[] - obsolete */
+#define RPMTAG_OLDSUGGESTS RPMTAG_OLDSUGGESTSNAME /* s[] - obsolete */
+    RPMTAG_OLDSUGGESTSVERSION	= 1157,	/* s[] - obsolete */
+    RPMTAG_OLDSUGGESTSFLAGS	= 1158,	/* i[] - obsolete */
+    RPMTAG_OLDENHANCESNAME	= 1159,	/* s[] - obsolete */
+#define RPMTAG_OLDENHANCES RPMTAG_OLDENHANCESNAME /* s[] - obsolete */
+    RPMTAG_OLDENHANCESVERSION	= 1160,	/* s[] - obsolete */
+    RPMTAG_OLDENHANCESFLAGS	= 1161,	/* i[] - obsolete */
     RPMTAG_PRIORITY		= 1162, /* i[] extension placeholder (unimplemented) */
     RPMTAG_CVSID		= 1163, /* s (unimplemented) */
 #define	RPMTAG_SVNID	RPMTAG_CVSID	/* s (unimplemented) */
@@ -268,15 +269,17 @@ typedef enum rpmTag_e {
     RPMTAG_OBSOLETEATTRSX	= 1190, /* i[] (unimplemented) */
     RPMTAG_PROVIDEATTRSX	= 1191, /* i[] (unimplemented) */
     RPMTAG_REQUIREATTRSX	= 1192, /* i[] (unimplemented) */
-    RPMTAG_BUILDPROVIDES	= 1193, /* internal */
-    RPMTAG_BUILDOBSOLETES	= 1194, /* internal */
+    RPMTAG_BUILDPROVIDES	= 1193, /* internal (unimplemented) */
+    RPMTAG_BUILDOBSOLETES	= 1194, /* internal (unimplemented) */
     RPMTAG_DBINSTANCE		= 1195, /* i extension */
     RPMTAG_NVRA			= 1196, /* s extension */
+
+    /* tags 1997-4999 reserved */
     RPMTAG_FILENAMES		= 5000, /* s[] extension */
     RPMTAG_FILEPROVIDE		= 5001, /* s[] extension */
     RPMTAG_FILEREQUIRE		= 5002, /* s[] extension */
-    RPMTAG_FSNAMES		= 5003, /* s[] extension */
-    RPMTAG_FSSIZES		= 5004, /* l[] extension */
+    RPMTAG_FSNAMES		= 5003, /* s[] (unimplemented) */
+    RPMTAG_FSSIZES		= 5004, /* l[] (unimplemented) */
     RPMTAG_TRIGGERCONDS		= 5005, /* s[] extension */
     RPMTAG_TRIGGERTYPE		= 5006, /* s[] extension */
     RPMTAG_ORIGFILENAMES	= 5007, /* s[] extension */
@@ -292,12 +295,119 @@ typedef enum rpmTag_e {
     RPMTAG_HEADERCOLOR		= 5017, /* i extension */
     RPMTAG_VERBOSE		= 5018, /* i extension */
     RPMTAG_EPOCHNUM		= 5019, /* i extension */
+    RPMTAG_PREINFLAGS		= 5020, /* i */
+    RPMTAG_POSTINFLAGS		= 5021, /* i */
+    RPMTAG_PREUNFLAGS		= 5022, /* i */
+    RPMTAG_POSTUNFLAGS		= 5023, /* i */
+    RPMTAG_PRETRANSFLAGS	= 5024, /* i */
+    RPMTAG_POSTTRANSFLAGS	= 5025, /* i */
+    RPMTAG_VERIFYSCRIPTFLAGS	= 5026, /* i */
+    RPMTAG_TRIGGERSCRIPTFLAGS	= 5027, /* i[] */
+    RPMTAG_COLLECTIONS		= 5029, /* s[] list of collections (unimplemented) */
+    RPMTAG_POLICYNAMES		= 5030,	/* s[] */
+    RPMTAG_POLICYTYPES		= 5031,	/* s[] */
+    RPMTAG_POLICYTYPESINDEXES	= 5032,	/* i[] */
+    RPMTAG_POLICYFLAGS		= 5033,	/* i[] */
+    RPMTAG_VCS			= 5034, /* s */
+    RPMTAG_ORDERNAME		= 5035,	/* s[] */
+    RPMTAG_ORDERVERSION		= 5036,	/* s[] */
+    RPMTAG_ORDERFLAGS		= 5037,	/* i[] */
+    RPMTAG_MSSFMANIFEST		= 5038, /* s[] reservation (unimplemented) */
+    RPMTAG_MSSFDOMAIN		= 5039, /* s[] reservation (unimplemented) */
+    RPMTAG_INSTFILENAMES	= 5040, /* s[] extension */
+    RPMTAG_REQUIRENEVRS		= 5041, /* s[] extension */
+    RPMTAG_PROVIDENEVRS		= 5042, /* s[] extension */
+    RPMTAG_OBSOLETENEVRS	= 5043, /* s[] extension */
+    RPMTAG_CONFLICTNEVRS	= 5044, /* s[] extension */
+    RPMTAG_FILENLINKS		= 5045,	/* i[] extension */
+    RPMTAG_RECOMMENDNAME	= 5046,	/* s[] */
+#define	RPMTAG_RECOMMENDS RPMTAG_RECOMMENDNAME	/* s[] */
+    RPMTAG_RECOMMENDVERSION	= 5047,	/* s[] */
+    RPMTAG_RECOMMENDFLAGS	= 5048,	/* i[] */
+    RPMTAG_SUGGESTNAME		= 5049,	/* s[] */
+#define	RPMTAG_SUGGESTS RPMTAG_SUGGESTNAME	/* s[] */
+    RPMTAG_SUGGESTVERSION	= 5050,	/* s[] */
+    RPMTAG_SUGGESTFLAGS		= 5051,	/* i[] */
+    RPMTAG_SUPPLEMENTNAME	= 5052,	/* s[] */
+#define	RPMTAG_SUPPLEMENTS RPMTAG_SUPPLEMENTNAME /* s[] */
+    RPMTAG_SUPPLEMENTVERSION	= 5053,	/* s[] */
+    RPMTAG_SUPPLEMENTFLAGS	= 5054,	/* i[] */
+    RPMTAG_ENHANCENAME		= 5055,	/* s[] */
+#define	RPMTAG_ENHANCES RPMTAG_ENHANCENAME	/* s[] */
+    RPMTAG_ENHANCEVERSION	= 5056,	/* s[] */
+    RPMTAG_ENHANCEFLAGS		= 5057,	/* i[] */
+    RPMTAG_RECOMMENDNEVRS	= 5058, /* s[] extension */
+    RPMTAG_SUGGESTNEVRS		= 5059, /* s[] extension */
+    RPMTAG_SUPPLEMENTNEVRS	= 5060, /* s[] extension */
+    RPMTAG_ENHANCENEVRS		= 5061, /* s[] extension */
+    RPMTAG_ENCODING		= 5062, /* s */
+    RPMTAG_FILETRIGGERIN		= 5063, /* internal */
+    RPMTAG_FILETRIGGERUN		= 5064, /* internal */
+    RPMTAG_FILETRIGGERPOSTUN		= 5065, /* internal */
+    RPMTAG_FILETRIGGERSCRIPTS		= 5066, /* s[] */
+    RPMTAG_FILETRIGGERSCRIPTPROG	= 5067, /* s[] */
+    RPMTAG_FILETRIGGERSCRIPTFLAGS	= 5068, /* i[] */
+    RPMTAG_FILETRIGGERNAME		= 5069, /* s[] */
+    RPMTAG_FILETRIGGERINDEX		= 5070, /* i[] */
+    RPMTAG_FILETRIGGERVERSION		= 5071, /* s[] */
+    RPMTAG_FILETRIGGERFLAGS		= 5072, /* i[] */
+    RPMTAG_TRANSFILETRIGGERIN		= 5073, /* internal */
+    RPMTAG_TRANSFILETRIGGERUN		= 5074, /* internal */
+    RPMTAG_TRANSFILETRIGGERPOSTUN	= 5075, /* internal */
+    RPMTAG_TRANSFILETRIGGERSCRIPTS	= 5076, /* s[] */
+    RPMTAG_TRANSFILETRIGGERSCRIPTPROG	= 5077, /* s[] */
+    RPMTAG_TRANSFILETRIGGERSCRIPTFLAGS	= 5078, /* i[] */
+    RPMTAG_TRANSFILETRIGGERNAME		= 5079, /* s[] */
+    RPMTAG_TRANSFILETRIGGERINDEX	= 5080, /* i[] */
+    RPMTAG_TRANSFILETRIGGERVERSION	= 5081, /* s[] */
+    RPMTAG_TRANSFILETRIGGERFLAGS	= 5082, /* i[] */
+    RPMTAG_REMOVEPATHPOSTFIXES  = 5083, /* s internal */
+    RPMTAG_FILETRIGGERPRIORITIES	= 5084, /* i[] */
+    RPMTAG_TRANSFILETRIGGERPRIORITIES	= 5085, /* i[] */
+    RPMTAG_FILETRIGGERCONDS		= 5086, /* s[] extension */
+    RPMTAG_FILETRIGGERTYPE		= 5087, /* s[] extension */
+    RPMTAG_TRANSFILETRIGGERCONDS	= 5088, /* s[] extension */
+    RPMTAG_TRANSFILETRIGGERTYPE		= 5089, /* s[] extension */
+    RPMTAG_FILESIGNATURES	= 5090, /* s[] */
+    RPMTAG_FILESIGNATURELENGTH  = 5091, /* i */
+    RPMTAG_PAYLOADDIGEST	= 5092, /* s[] */
+    RPMTAG_PAYLOADDIGESTALGO	= 5093, /* i */
+    RPMTAG_AUTOINSTALLED	= 5094, /* i reservation (unimplemented) */
+    RPMTAG_IDENTITY		= 5095, /* s reservation (unimplemented) */
+    RPMTAG_MODULARITYLABEL	= 5096, /* s */
+    RPMTAG_PAYLOADDIGESTALT	= 5097, /* s[] */
 
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
 } rpmTag;
 
 #define	RPMTAG_EXTERNAL_TAG		1000000
-#define RPMTAG_NOT_FOUND		-1
+
+/** \ingroup rpmtag
+ * Rpm database index tags.
+ */
+typedef enum rpmDbiTag_e {
+    RPMDBI_PACKAGES		= 0,	/* Installed package headers. */
+    RPMDBI_LABEL		= 2,	/* NEVRA label pseudo index */
+    RPMDBI_NAME			= RPMTAG_NAME,
+    RPMDBI_BASENAMES		= RPMTAG_BASENAMES,
+    RPMDBI_GROUP		= RPMTAG_GROUP,
+    RPMDBI_REQUIRENAME		= RPMTAG_REQUIRENAME,
+    RPMDBI_PROVIDENAME		= RPMTAG_PROVIDENAME,
+    RPMDBI_CONFLICTNAME		= RPMTAG_CONFLICTNAME,
+    RPMDBI_OBSOLETENAME		= RPMTAG_OBSOLETENAME,
+    RPMDBI_TRIGGERNAME		= RPMTAG_TRIGGERNAME,
+    RPMDBI_DIRNAMES		= RPMTAG_DIRNAMES,
+    RPMDBI_INSTALLTID		= RPMTAG_INSTALLTID,
+    RPMDBI_SIGMD5		= RPMTAG_SIGMD5,
+    RPMDBI_SHA1HEADER		= RPMTAG_SHA1HEADER,
+    RPMDBI_INSTFILENAMES	= RPMTAG_INSTFILENAMES,
+    RPMDBI_FILETRIGGERNAME	= RPMTAG_FILETRIGGERNAME,
+    RPMDBI_TRANSFILETRIGGERNAME = RPMTAG_TRANSFILETRIGGERNAME,
+    RPMDBI_RECOMMENDNAME	= RPMTAG_RECOMMENDNAME,
+    RPMDBI_SUGGESTNAME		= RPMTAG_SUGGESTNAME,
+    RPMDBI_SUPPLEMENTNAME	= RPMTAG_SUPPLEMENTNAME,
+    RPMDBI_ENHANCENAME		= RPMTAG_ENHANCENAME,
+} rpmDbiTag;
 
 /** \ingroup signature
  * Tags found in signature header from package.
@@ -311,13 +421,19 @@ typedef enum rpmSigTag_e {
     RPMSIGTAG_GPG	= 1005, /*!< internal GnuPG signature. */
     RPMSIGTAG_PGP5	= 1006,	/*!< internal PGP5 signature @deprecated legacy. */
     RPMSIGTAG_PAYLOADSIZE = 1007,/*!< internal uncompressed payload size (32bit) in bytes. */
+    RPMSIGTAG_RESERVEDSPACE = 1008,/*!< internal space reserved for signatures */
     RPMSIGTAG_BADSHA1_1	= RPMTAG_BADSHA1_1,	/*!< internal Broken SHA1, take 1. */
     RPMSIGTAG_BADSHA1_2	= RPMTAG_BADSHA1_2,	/*!< internal Broken SHA1, take 2. */
-    RPMSIGTAG_SHA1	= RPMTAG_SHA1HEADER,	/*!< internal sha1 header digest. */
     RPMSIGTAG_DSA	= RPMTAG_DSAHEADER,	/*!< internal DSA header signature. */
     RPMSIGTAG_RSA	= RPMTAG_RSAHEADER,	/*!< internal RSA header signature. */
+    RPMSIGTAG_SHA1	= RPMTAG_SHA1HEADER,	/*!< internal sha1 header digest. */
     RPMSIGTAG_LONGSIZE	= RPMTAG_LONGSIGSIZE,	/*!< internal Header+Payload size (64bit) in bytes. */
     RPMSIGTAG_LONGARCHIVESIZE = RPMTAG_LONGARCHIVESIZE, /*!< internal uncompressed payload size (64bit) in bytes. */
+    RPMSIGTAG_SHA256	= RPMTAG_SHA256HEADER,
+    RPMSIGTAG_FILESIGNATURES		= RPMTAG_SIG_BASE + 18,
+    RPMSIGTAG_FILESIGNATURELENGTH	= RPMTAG_SIG_BASE + 19,
+    RPMSIGTAG_VERITYSIGNATURES		= RPMTAG_VERITYSIGNATURES,
+    RPMSIGTAG_VERITYSIGNATUREALGO	= RPMTAG_VERITYSIGNATUREALGO,
 } rpmSigTag;
 
 
@@ -325,7 +441,7 @@ typedef enum rpmSigTag_e {
  * The basic types of data in tags from headers.
  */
 typedef enum rpmTagType_e {
-#define	RPM_MIN_TYPE		0
+#define	RPM_MIN_TYPE		1
     RPM_NULL_TYPE		=  0,
     RPM_CHAR_TYPE		=  1,
     RPM_INT8_TYPE		=  2,
@@ -351,62 +467,60 @@ typedef enum rpmTagClass_e {
     RPM_BINARY_CLASS	= 3,
 } rpmTagClass;
 
-/** \ingroup header
- * New rpm data types under consideration/development.
- * These data types may (or may not) be added to rpm at some point. In order
- * to avoid incompatibility with legacy versions of rpm, these data (sub-)types
- * are introduced into the header by overloading RPM_BIN_TYPE, with the binary
- * value of the tag a 16 byte image of what should/will be in the header index,
- * followed by per-tag private data.
- */
-typedef enum rpmSubTagType_e {
-    RPM_REGION_TYPE		= -10,
-    RPM_BIN_ARRAY_TYPE		= -11,
-  /*!<@todo Implement, kinda like RPM_STRING_ARRAY_TYPE for known (but variable)
-	length binary data. */
-    RPM_XREF_TYPE		= -12
-  /*!<@todo Implement, intent is to to carry a (???,tagNum,valNum) cross
-	reference to retrieve data from other tags. */
-} rpmSubTagType;
-
 /** \ingroup header
  *  * Identify how to return the header data type.
  *   */
-typedef enum rpmTagReturnType_e {
+enum rpmTagReturnType_e {
     RPM_ANY_RETURN_TYPE         = 0,
     RPM_SCALAR_RETURN_TYPE      = 0x00010000,
     RPM_ARRAY_RETURN_TYPE       = 0x00020000,
     RPM_MAPPING_RETURN_TYPE     = 0x00040000,
     RPM_MASK_RETURN_TYPE        = 0xffff0000
-} rpmTagReturnType;
+};
+
+typedef rpmFlags rpmTagReturnType;
 
 /** \ingroup rpmtag
  * Return tag name from value.
  * @param tag		tag value
  * @return		tag name, "(unknown)" on not found
  */
-const char * rpmTagGetName(rpmTag tag);
+const char * rpmTagGetName(rpmTagVal tag);
+
+/** \ingroup rpmtag
+ * Return tag data type from value.
+ * @param tag		tag value
+ * @return		tag data type + return type, RPM_NULL_TYPE on not found.
+ */
+rpmTagType rpmTagGetType(rpmTagVal tag);
 
 /** \ingroup rpmtag
  * Return tag data type from value.
  * @param tag		tag value
  * @return		tag data type, RPM_NULL_TYPE on not found.
  */
-rpmTagType rpmTagGetType(rpmTag tag);
+rpmTagType rpmTagGetTagType(rpmTagVal tag);
+
+/** \ingroup rpmtag
+ * Return tag data type from value.
+ * @param tag		tag value
+ * @return		tag data return type, RPM_NULL_TYPE on not found.
+ */
+rpmTagReturnType rpmTagGetReturnType(rpmTagVal tag);
 
 /** \ingroup rpmtag
  * Return tag data class from value.
  * @param tag		tag value
  * @return		tag data class, RPM_NULL_CLASS on not found.
  */
-rpmTagClass rpmTagGetClass(rpmTag tag);
+rpmTagClass rpmTagGetClass(rpmTagVal tag);
 
 /** \ingroup rpmtag
  * Return tag value from name.
  * @param tagstr	name of tag
  * @return		tag value, -1 on not found
  */
-rpmTag rpmTagGetValue(const char * tagstr);
+rpmTagVal rpmTagGetValue(const char * tagstr);
 
 /** \ingroup rpmtag
  * Return data class of type
@@ -417,7 +531,7 @@ rpmTagClass rpmTagTypeGetClass(rpmTagType type);
 
 /** \ingroup rpmtag
  * Return known rpm tag names, sorted by name.
- * @retval tagnames 	tag container of string array type
+ * @param[out] tagnames	tag container of string array type
  * @param fullname	return short or full name
  * @return		number of tag names, 0 on error
  */
