@@ -1,6 +1,12 @@
 #ifndef _RPMTD_H
 #define _RPMTD_H
 
+/** \ingroup rpmtd
+ *  \file lib/rpmtd.h
+ *
+ *  RPM Tag Data Container API
+ */
+
 #include <rpm/rpmtypes.h>
 #include <rpm/argv.h>
 
@@ -30,6 +36,7 @@ struct rpmtd_s {
     rpm_data_t data;	/* pointer to actual data */
     rpmtdFlags flags;	/* flags on memory allocation etc */
     int ix;		/* iteration index */
+    rpm_count_t size;	/* size of data (only works for RPMTD_IMMUTABLE atm) */
 };
 
 /** \ingroup rpmtd
@@ -66,6 +73,14 @@ void rpmtdFreeData(rpmtd td);
  */
 rpm_count_t rpmtdCount(rpmtd td);
 
+/** \ingroup rpmtd
+ * Retrieve container data size (eg required for allocation).
+ * Note this currently only works for RPMTD_IMMUTABLE data.
+ * @param td		Tag data container
+ * @return		Data size in bytes.
+ */
+rpm_count_t rpmtdSize(rpmtd td);
+
 /** \ingroup rpmtd
  * Retrieve tag of the container.
  * @param td		Tag data container
@@ -228,6 +243,10 @@ typedef enum rpmtdFormats_e {
     RPMTD_FORMAT_VFLAGS		= 17,	/* file verify flags (int types) */
     RPMTD_FORMAT_EXPAND		= 18,	/* macro expansion (string types) */
     RPMTD_FORMAT_FSTATUS	= 19,	/* file verify status (int types) */
+    RPMTD_FORMAT_HUMANSI	= 20,	/* human readable value, K = 1000 (int types) */
+    RPMTD_FORMAT_HUMANIEC	= 21,	/* human readable value, K = 1024 (int types) */
+    RPMTD_FORMAT_TAGNAME	= 22,	/* tag name (any type) */
+    RPMTD_FORMAT_TAGNUM		= 23,	/* tag number (any type) */
 } rpmtdFormats;
 
 /** \ingroup rpmtd
@@ -346,7 +365,7 @@ int rpmtdFromArgv(rpmtd td, rpmTagVal tag, ARGV_t argv);
  */
 int rpmtdFromArgi(rpmtd td, rpmTagVal tag, ARGI_t argi);
 
-/* \ingroup rpmtd
+/** \ingroup rpmtd
  * Perform deep copy of container.
  * Create a modifiable copy of tag data container (on string arrays each
  * string is separately allocated)
@@ -356,6 +375,14 @@ int rpmtdFromArgi(rpmtd td, rpmTagVal tag, ARGI_t argi);
  */
 rpmtd rpmtdDup(rpmtd td);
 
+/** \ingroup rpmtd
+ * Push string array container contents to a string pool, return string ids.
+ * @param td		Tag data container
+ * @param pool		String pool
+ * @return		Array of string id's (malloced)
+ */
+rpmsid * rpmtdToPool(rpmtd td, rpmstrPool pool);
+
 #ifdef __cplusplus
 }
 #endif
