@@ -6,6 +6,18 @@
 
 #include <inttypes.h>
 #include <libgen.h>
+#include <errno.h>
+#include <sys/statvfs.h>
+#include <fcntl.h>
+
+/* duplicated from cpio.c */
+#if MAJOR_IN_MKDEV
+#include <sys/mkdev.h>
+#elif MAJOR_IN_SYSMACROS
+#include <sys/sysmacros.h>
+#else
+#include <sys/types.h> /* already included from system.h */
+#endif
 
 #include <rpm/rpmlib.h>		/* rpmMachineScore, rpmReadPackageFile */
 #include <rpm/rpmmacro.h>	/* XXX for rpmExpand */
@@ -31,25 +43,6 @@
 
 #include "lib/rpmplugins.h"
 
-/* XXX FIXME: merge with existing (broken?) tests in system.h */
-/* portability fiddles */
-#if STATFS_IN_SYS_STATVFS
-#include <sys/statvfs.h>
-
-#else
-# if STATFS_IN_SYS_VFS
-#  include <sys/vfs.h>
-# else
-#  if STATFS_IN_SYS_MOUNT
-#   include <sys/mount.h>
-#  else
-#   if STATFS_IN_SYS_STATFS
-#    include <sys/statfs.h>
-#   endif
-#  endif
-# endif
-#endif
-
 #include "debug.h"
 
 struct diskspaceInfo_s {
@@ -64,6 +57,8 @@ struct diskspaceInfo_s {
     int64_t oineeded;	/*!< Bookkeeping to avoid duplicate reports */
     int64_t bdelta;	/*!< Delta for temporary space need on updates */
     int64_t idelta;	/*!< Delta for temporary inode need on updates */
+
+    int rotational;	/*!< Rotational media? */
 };
 
 /* Adjust for root only reserved space. On linux e2fs, this is 5%. */
@@ -109,10 +104,29 @@ static char *getMntPoint(const char *dirName, dev_t dev)
     return res;
 }
 
+static int getRotational(const char *dirName, dev_t dev)
+{
+    int rotational = 1;	/* Be a good pessimist, assume the worst */
+#if defined(__linux__)
+    char *devpath = NULL;
+    FILE *f = NULL;
+
+    rasprintf(&devpath, "/sys/dev/block/%d:%d/queue/rotational",
+			major(dev), minor(dev));
+    if ((f = fopen(devpath, "r")) != NULL) {
+	int v;
+	if (fscanf(f, "%d", &v) == 1)
+	    rotational = (v == 1);
+	fclose(f);
+    }
+    free(devpath);
+#endif
+
+    return rotational;
+}
+
 static int rpmtsInitDSI(const rpmts ts)
 {
-    if (rpmtsFilterFlags(ts) & RPMPROB_FILTER_DISKSPACE)
-	return 0;
     ts->dsi = _free(ts->dsi);
     ts->dsi = xcalloc(1, sizeof(*ts->dsi));
     return 0;
@@ -124,25 +138,10 @@ static rpmDiskSpaceInfo rpmtsCreateDSI(const rpmts ts, dev_t dev,
     rpmDiskSpaceInfo dsi;
     struct stat sb;
     int rc;
-
-#if STATFS_IN_SYS_STATVFS
     struct statvfs sfb;
+
     memset(&sfb, 0, sizeof(sfb));
     rc = statvfs(dirName, &sfb);
-#else
-    struct statfs sfb;
-    memset(&sfb, 0, sizeof(sfb));
-#  if STAT_STATFS4
-/* This platform has the 4-argument version of the statfs call.  The last two
- * should be the size of struct statfs and 0, respectively.  The 0 is the
- * filesystem type, and is always 0 when statfs is called on a mounted
- * filesystem, as we're doing.
- */
-    rc = statfs(dirName, &sfb, sizeof(sfb), 0);
-#  else
-    rc = statfs(dirName, &sfb);
-#  endif
-#endif
     if (rc)
 	return NULL;
 
@@ -162,15 +161,8 @@ static rpmDiskSpaceInfo rpmtsCreateDSI(const rpmts ts, dev_t dev,
 	dsi->bsize = 512;       /* we need a bsize */
     dsi->bneeded = 0;
     dsi->ineeded = 0;
-#ifdef STATFS_HAS_F_BAVAIL
     dsi->bavail = (sfb.f_flag & ST_RDONLY) ? 0 : sfb.f_bavail;
-#else
-/* FIXME: the statfs struct doesn't have a member to tell how many blocks are
- * available for non-superusers.  f_blocks - f_bfree is probably too big, but
- * it's about all we can do.
- */
-    dsi->bavail = sfb.f_blocks - sfb.f_bfree;
-#endif
+
     /* XXX Avoid FAT and other file systems that have not inodes. */
     /* XXX assigning negative value to unsigned type */
     dsi->iavail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
@@ -179,6 +171,9 @@ static rpmDiskSpaceInfo rpmtsCreateDSI(const rpmts ts, dev_t dev,
     /* Find mount point belonging to this device number */
     dsi->mntPoint = getMntPoint(dirName, dsi->dev);
 
+    /* Initialized on demand */
+    dsi->rotational = -1;
+
     /* normalize block size to 4096 bytes if it is too big. */
     if (dsi->bsize > 4096) {
 	uint64_t old_size = dsi->bavail * dsi->bsize;
@@ -190,9 +185,9 @@ static rpmDiskSpaceInfo rpmtsCreateDSI(const rpmts ts, dev_t dev,
     }
 
     rpmlog(RPMLOG_DEBUG,
-	   "0x%08x %8" PRId64 " %12" PRId64 " %12" PRId64" %s\n",
+	   "0x%08x %8" PRId64 " %12" PRId64 " %12" PRId64" rotational:%d %s\n",
 	   (unsigned) dsi->dev, dsi->bsize,
-	   dsi->bavail, dsi->iavail,
+	   dsi->bavail, dsi->iavail, dsi->rotational,
 	   dsi->mntPoint);
     return dsi;
 }
@@ -212,6 +207,22 @@ static rpmDiskSpaceInfo rpmtsGetDSI(const rpmts ts, dev_t dev,
     return dsi;
 }
 
+static int rpmtsGetDSIRotational(rpmts ts)
+{
+    int rc = 1;
+    int nrot = 0;
+    rpmDiskSpaceInfo dsi = ts->dsi;
+    while (dsi && dsi->bsize != 0) {
+	if (dsi->rotational < 0)
+	    dsi->rotational = getRotational(dsi->mntPoint, dsi->dev);
+	nrot += dsi->rotational;
+	dsi++;
+    }
+    if (dsi != ts->dsi && nrot == 0)
+	rc = 0;
+    return rc;
+}
+
 static void rpmtsUpdateDSI(const rpmts ts, dev_t dev, const char *dirName,
 		rpm_loff_t fileSize, rpm_loff_t prevSize, rpm_loff_t fixupSize,
 		rpmFileAction action)
@@ -481,13 +492,6 @@ static void handleInstInstalledFile(const rpmts ts, rpmte p, rpmfiles fi, int fx
 	rpmfsSetAction(fs, fx, action);
     }
 
-    /* Skip already existing files - if 'minimize_writes' is set. */
-    if ((!isCfgFile) && (rpmfsGetAction(fs, fx) == FA_UNKNOWN)  && ts->min_writes) {
-	if (rpmfileContentsEqual(otherFi, ofx, fi, fx)) {
-	   rpmfsSetAction(fs, fx, FA_TOUCH);
-	}
-    }
-
     otherFileSize = rpmfilesFSize(otherFi, ofx);
 
     /* Only account for the last file of a hardlink set */
@@ -497,6 +501,16 @@ static void handleInstInstalledFile(const rpmts ts, rpmte p, rpmfiles fi, int fx
 
     /* Add one to make sure the size is not zero */
     rpmfilesSetFReplacedSize(fi, fx, otherFileSize + 1);
+
+    /* Just touch already existing files if minimize_writes is enabled */
+    if (ts->min_writes) {
+	if ((!isCfgFile) && (rpmfsGetAction(fs, fx) == FA_UNKNOWN)) {
+	    /* XXX fsm can't handle FA_TOUCH of hardlinked files */
+	    int nolinks = (nlink == 1 && rpmfilesFNlink(fi, fx) == 1);
+	    if (nolinks && rpmfileContentsEqual(otherFi, ofx, fi, fx))
+	       rpmfsSetAction(fs, fx, FA_TOUCH);
+	}
+    }
 }
 
 /**
@@ -689,6 +703,8 @@ assert(otherFi != NULL);
 	    /* Otherwise, we can just erase. */
 	    rpmfsSetAction(fs, i, FA_ERASE);
 	    break;
+	default:
+	    break;
 	}
 	rpmfilesFree(otherFi);
 
@@ -720,7 +736,7 @@ static void ensureOlder(rpmstrPool tspool, const rpmte p, const Header h)
 
     req = rpmdsSinglePool(tspool, RPMTAG_REQUIRENAME,
 			  rpmteN(p), rpmteEVR(p), reqFlags);
-    if (rpmdsMatches(tspool, h, -1, req, 1, _rpmds_nopromote) == 0) {
+    if (rpmdsMatches(tspool, h, -1, req, 1) == 0) {
 	char * altNEVR = headerGetAsString(h, RPMTAG_NEVRA);
 	rpmteAddProblem(p, RPMPROB_OLDPACKAGE, altNEVR, NULL,
 			headerGetInstance(h));
@@ -809,6 +825,7 @@ static void skipInstallFiles(const rpmts ts, rpmfiles files, rpmfs fs)
     rpm_color_t FColor;
     int noConfigs = (rpmtsFlags(ts) & RPMTRANS_FLAG_NOCONFIGS);
     int noDocs = (rpmtsFlags(ts) & RPMTRANS_FLAG_NODOCS);
+    int noArtifacts = (rpmtsFlags(ts) & RPMTRANS_FLAG_NOARTIFACTS);
     int * drc;
     char * dff;
     int dc;
@@ -908,6 +925,13 @@ static void skipInstallFiles(const rpmts ts, rpmfiles files, rpmfs fs)
 	    rpmfsSetAction(fs, i, FA_SKIPNSTATE);
 	    continue;
 	}
+
+	/* Skip artifacts if requested. */
+	if (noArtifacts && (rpmfiFFlags(fi) & RPMFILE_ARTIFACT)) {
+	    drc[ix]--;	dff[ix] = 1;
+	    rpmfsSetAction(fs, i, FA_SKIPNSTATE);
+	    continue;
+	}
     }
 
     /* Skip (now empty) directories that had skipped files. */
@@ -1156,6 +1180,8 @@ void checkInstalledFiles(rpmts ts, uint64_t fileCount, fingerPrintCache fpc)
 			    rpmfsSetAction(fs, recs[j].fileno, FA_SKIP);
 		    }
 		    break;
+		default:
+		    break;
 		}
 		rpmfilesFree(fi);
 	    }
@@ -1195,12 +1221,21 @@ static rpm_loff_t countPkgs(rpmts ts, rpmElementTypes types)
 
 struct vfydata_s {
     char *msg;
+    int type[3];
     int vfylevel;
 };
 
 static int vfyCb(struct rpmsinfo_s *sinfo, void *cbdata)
 {
     struct vfydata_s *vd = cbdata;
+
+    if (sinfo->type & RPMSIG_VERIFIABLE_TYPE && sinfo->rc != RPMRC_NOTFOUND) {
+	int res = (sinfo->rc != RPMRC_OK);
+	/* Take care not to override a previous failure with success */
+	if (res > vd->type[sinfo->type])
+	    vd->type[sinfo->type] = res;
+    }
+
     switch (sinfo->rc) {
     case RPMRC_OK:
 	break;
@@ -1215,8 +1250,10 @@ static int vfyCb(struct rpmsinfo_s *sinfo, void *cbdata)
 	 */
 	if (!(vd->vfylevel & RPMSIG_SIGNATURE_TYPE))
 	    sinfo->rc = RPMRC_OK;
+	/* fallthrough */
     default:
-	vd->msg = rpmsinfoMsg(sinfo);
+	if (sinfo->rc)
+	    vd->msg = rpmsinfoMsg(sinfo);
 	break;
     }
     return (sinfo->rc == 0);
@@ -1241,8 +1278,10 @@ static int verifyPackageFiles(rpmts ts, rpm_loff_t total)
 	struct rpmvs_s *vs = rpmvsCreate(vfylevel, vsflags, keyring);
 	struct vfydata_s vd = {
 	    .msg = NULL,
+	    .type = { -1, -1, -1, },
 	    .vfylevel = vfylevel,
 	};
+	int verified = 0;
 	rpmRC prc = RPMRC_FAIL;
 
 	rpmtsNotify(ts, p, RPMCALLBACK_VERIFY_PROGRESS, oc++, total);
@@ -1255,6 +1294,13 @@ static int verifyPackageFiles(rpmts ts, rpm_loff_t total)
 	if (prc == RPMRC_OK)
 	    prc = rpmvsVerify(vs, RPMSIG_VERIFIABLE_TYPE, vfyCb, &vd);
 
+	/* Record verify result */
+	if (vd.type[RPMSIG_SIGNATURE_TYPE] == RPMRC_OK)
+	    verified |= RPMSIG_SIGNATURE_TYPE;
+	if (vd.type[RPMSIG_DIGEST_TYPE] == RPMRC_OK)
+	    verified |= RPMSIG_DIGEST_TYPE;
+	rpmteSetVerified(p, verified);
+
 	if (prc)
 	    rpmteAddProblem(p, RPMPROB_VERIFY, NULL, vd.msg, 0);
 
@@ -1270,6 +1316,64 @@ static int verifyPackageFiles(rpmts ts, rpm_loff_t total)
     return rc;
 }
 
+static void checkAdded(rpmts ts, rpmprobFilterFlags probFilter, rpmte p)
+{
+    if (!(probFilter & RPMPROB_FILTER_IGNOREARCH) && badArch(rpmteA(p)))
+	rpmteAddProblem(p, RPMPROB_BADARCH, rpmteA(p), NULL, 0);
+
+    if (!(probFilter & RPMPROB_FILTER_IGNOREOS) && badOs(rpmteO(p)))
+	rpmteAddProblem(p, RPMPROB_BADOS, rpmteO(p), NULL, 0);
+
+    if (!(probFilter & RPMPROB_FILTER_OLDPACKAGE)) {
+	rpmstrPool tspool = rpmtsPool(ts);
+	Header h;
+	rpmdbMatchIterator mi;
+	mi = rpmtsInitIterator(ts, RPMDBI_NAME, rpmteN(p), 0);
+	while ((h = rpmdbNextIterator(mi)) != NULL)
+	    ensureOlder(tspool, p, h);
+	rpmdbFreeIterator(mi);
+    }
+
+    if (!(probFilter & RPMPROB_FILTER_REPLACEPKG) && rpmteAddOp(p) != RPMTE_REINSTALL) {
+	Header h;
+	rpmdbMatchIterator mi;
+	mi = rpmtsInitIterator(ts, RPMDBI_NAME, rpmteN(p), 0);
+	rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP, rpmteE(p));
+	rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_STRCMP, rpmteV(p));
+	rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP, rpmteR(p));
+	if (rpmtsColor(ts)) {
+	    rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_STRCMP, rpmteA(p));
+	    rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_STRCMP, rpmteO(p));
+	}
+
+	if ((h = rpmdbNextIterator(mi)) != NULL) {
+	    rpmteAddProblem(p, RPMPROB_PKG_INSTALLED, NULL, NULL,
+			    headerGetInstance(h));
+	}
+	rpmdbFreeIterator(mi);
+    }
+
+    if (!(probFilter & RPMPROB_FILTER_FORCERELOCATE))
+	rpmteAddRelocProblems(p);
+}
+
+static void checkRemoved(rpmts ts, rpmprobFilterFlags probFilter, rpmte p)
+{
+    unsigned int offset = rpmteDBInstance(p);
+    int found = 0;
+    rpmdbMatchIterator mi;
+
+    mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, &offset, sizeof(offset));
+    while (rpmdbNextIterator(mi)) {
+	found = 1;
+	break;
+    }
+    rpmdbFreeIterator(mi);
+
+    if (!found)
+	rpmteAddProblem(p, RPMPROB_PKG_INSTALLED, NULL, NULL, 0);
+}
+
 /*
  * For packages being installed:
  * - verify package arch/os.
@@ -1277,10 +1381,10 @@ static int verifyPackageFiles(rpmts ts, rpm_loff_t total)
  */
 static rpmps checkProblems(rpmts ts)
 {
-    rpm_color_t tscolor = rpmtsColor(ts);
     rpmprobFilterFlags probFilter = rpmtsFilterFlags(ts);
-    rpmstrPool tspool = rpmtsPool(ts);
-    rpm_loff_t npkgs = countPkgs(ts, TR_ADDED);
+    rpmElementTypes etypes = (TR_ADDED|TR_REMOVED);
+    rpm_loff_t npkgs = countPkgs(ts, etypes);
+    rpm_loff_t ninst = 0;
     rpmtsi pi;
     rpmte p;
 
@@ -1291,49 +1395,23 @@ static rpmps checkProblems(rpmts ts)
     /* XXX Only added packages need be checked. */
     rpmlog(RPMLOG_DEBUG, "sanity checking %lu elements\n", npkgs);
     pi = rpmtsiInit(ts);
-    while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
-
-	if (!(probFilter & RPMPROB_FILTER_IGNOREARCH) && badArch(rpmteA(p)))
-	    rpmteAddProblem(p, RPMPROB_BADARCH, rpmteA(p), NULL, 0);
-
-	if (!(probFilter & RPMPROB_FILTER_IGNOREOS) && badOs(rpmteO(p)))
-	    rpmteAddProblem(p, RPMPROB_BADOS, rpmteO(p), NULL, 0);
-
-	if (!(probFilter & RPMPROB_FILTER_OLDPACKAGE)) {
-	    Header h;
-	    rpmdbMatchIterator mi;
-	    mi = rpmtsInitIterator(ts, RPMDBI_NAME, rpmteN(p), 0);
-	    while ((h = rpmdbNextIterator(mi)) != NULL)
-		ensureOlder(tspool, p, h);
-	    rpmdbFreeIterator(mi);
+    while ((p = rpmtsiNext(pi, etypes)) != NULL) {
+	switch (rpmteType(p)) {
+	case TR_ADDED:
+	    checkAdded(ts, probFilter, p);
+	    ninst++;
+	    break;
+	case TR_REMOVED:
+	    checkRemoved(ts, probFilter, p);
+	    break;
+	default:
+	    break;
 	}
-
-	if (!(probFilter & RPMPROB_FILTER_REPLACEPKG)) {
-	    Header h;
-	    rpmdbMatchIterator mi;
-	    mi = rpmtsPrunedIterator(ts, RPMDBI_NAME, rpmteN(p), 1);
-	    rpmdbSetIteratorRE(mi, RPMTAG_EPOCH, RPMMIRE_STRCMP, rpmteE(p));
-	    rpmdbSetIteratorRE(mi, RPMTAG_VERSION, RPMMIRE_STRCMP, rpmteV(p));
-	    rpmdbSetIteratorRE(mi, RPMTAG_RELEASE, RPMMIRE_STRCMP, rpmteR(p));
-	    if (tscolor) {
-		rpmdbSetIteratorRE(mi, RPMTAG_ARCH, RPMMIRE_STRCMP, rpmteA(p));
-		rpmdbSetIteratorRE(mi, RPMTAG_OS, RPMMIRE_STRCMP, rpmteO(p));
-	    }
-
-	    if ((h = rpmdbNextIterator(mi)) != NULL) {
-		rpmteAddProblem(p, RPMPROB_PKG_INSTALLED, NULL, NULL,
-				headerGetInstance(h));
-	    }
-	    rpmdbFreeIterator(mi);
-	}
-
-	if (!(probFilter & RPMPROB_FILTER_FORCERELOCATE))
-	    rpmteAddRelocProblems(p);
     }
     rpmtsiFree(pi);
 
-    if (rpmtsVfyLevel(ts) && !(probFilter & RPMPROB_FILTER_VERIFY))
-	verifyPackageFiles(ts, npkgs);
+    if (ninst > 0 && rpmtsVfyLevel(ts) && !(probFilter & RPMPROB_FILTER_VERIFY))
+	verifyPackageFiles(ts, ninst);
 
 exit:
     return rpmtsProblems(ts);
@@ -1393,8 +1471,41 @@ static int rpmtsSetup(rpmts ts, rpmprobFilterFlags ignoreSet)
     return 0;
 }
 
+/* XXX _minimize_writes is not safe for mass-consumption yet */
+#if 0
+/* Push a macro with current value or default if no previous value exists */
+static void ensureMacro(const char *name, const char *def)
+{
+    char *s = rpmExpand("%{?", name, "}", NULL);
+
+    if (*s != '\0' && atoi(s) >= 0)
+	def = s;
+
+    rpmPushMacro(NULL, name, NULL, def, 0);
+    free(s);
+}
+
+/* Enable / disable optimizations for solid state disks */
+static void setSSD(int enable)
+{
+    if (enable) {
+	rpmlog(RPMLOG_DEBUG, "optimizing for non-rotational disks\n");
+	ensureMacro("_minimize_writes", "1");
+    } else {
+	rpmPopMacro(NULL, "_minimize_writes");
+    }
+}
+#else
+static void setSSD(int enable)
+{
+}
+#endif
+
 static int rpmtsFinish(rpmts ts)
 {
+    if (rpmtsGetDSIRotational(ts) == 0)
+	setSSD(0);
+    rpmtsFreeDSI(ts);
     return rpmChrootSet(NULL);
 }
 
@@ -1470,7 +1581,8 @@ static int rpmtsPrepare(rpmts ts)
 			       hsize, 0, 0, FA_CREATE);
 	    }
 
-	    rpmtsCheckDSIProblems(ts, p);
+	    if (!(rpmtsFilterFlags(ts) & RPMPROB_FILTER_DISKSPACE))
+		rpmtsCheckDSIProblems(ts, p);
 	}
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_FINGERPRINT), 0);
 	rpmfilesFree(files);
@@ -1491,9 +1603,11 @@ static int rpmtsPrepare(rpmts ts)
 	rpmtsiFree(pi);
     }
 
+    if (rpmtsGetDSIRotational(ts) == 0)
+	setSSD(1);
+
 exit:
     fpCacheFree(fpc);
-    rpmtsFreeDSI(ts);
     return rc;
 }
 
@@ -1577,14 +1691,14 @@ rpmRC runScript(rpmts ts, rpmte te, Header h, ARGV_const_t prefixes,
     rpmRC stoprc, rc = RPMRC_OK;
     rpmTagVal stag = rpmScriptTag(script);
     FD_t sfd = NULL;
-    int warn_only = (stag != RPMTAG_PREIN &&
-		     stag != RPMTAG_PREUN &&
-		     stag != RPMTAG_PRETRANS &&
-		     stag != RPMTAG_VERIFYSCRIPT);
+    int warn_only = !(rpmScriptFlags(script) & RPMSCRIPT_FLAG_CRITICAL);
 
-    /* Fake up a transaction element for triggers from rpmdb */
+    if (rpmChrootIn())
+	return RPMRC_FAIL;
+
+    /* Create a temporary transaction element for triggers from rpmdb */
     if (te == NULL) {
-	te = rpmteNew(ts, h, TR_REMOVED, NULL, NULL);
+	te = rpmteNew(ts, h, TR_RPMDB, NULL, NULL, 0);
 	rpmteSetHeader(te, h);
     }
 
@@ -1594,7 +1708,7 @@ rpmRC runScript(rpmts ts, rpmte te, Header h, ARGV_const_t prefixes,
 
     rpmswEnter(rpmtsOp(ts, RPMTS_OP_SCRIPTLETS), 0);
     rc = rpmScriptRun(script, arg1, arg2, sfd,
-		      prefixes, warn_only, rpmtsPlugins(ts));
+		      prefixes, rpmtsPlugins(ts));
     rpmswExit(rpmtsOp(ts, RPMTS_OP_SCRIPTLETS), 0);
 
     /* Map warn-only errors to "notfound" for script stop callback */
@@ -1613,15 +1727,38 @@ rpmRC runScript(rpmts ts, rpmte te, Header h, ARGV_const_t prefixes,
 	rpmtsNotify(ts, te, RPMCALLBACK_SCRIPT_ERROR, stag, rc);
     }
 
+    rpmChrootOut();
+
     if (te != xte)
 	rpmteFree(te);
 
     return rc;
 }
 
+static void rpmtsSync(rpmts ts)
+{
+    if (rpmChrootDone())
+	return;
+
+#if HAVE_SYNCFS
+    for (rpmDiskSpaceInfo dsi = ts->dsi; dsi->bsize; dsi++) {
+	int fd = open(dsi->mntPoint, O_RDONLY);
+	if (fd != -1) {
+	    rpmlog(RPMLOG_DEBUG, "syncing fs %s\n", dsi->mntPoint);
+	    syncfs(fd);
+	    close(fd);
+	}
+    }
+#else
+    rpmlog(RPMLOG_DEBUG, "syncing all filesystems\n");
+    sync();
+#endif
+}
+
 int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 {
     int rc = -1; /* assume failure */
+    int nfailed = -1;
     tsMembers tsmem = rpmtsMembers(ts);
     rpmtxn txn = NULL;
     rpmps tsprobs = NULL;
@@ -1697,16 +1834,14 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 	tsmem->pool = rpmstrPoolFree(tsmem->pool);
 
     /* Run %transfiletriggerun scripts unless disabled */
-    if (!(rpmtsFlags(ts) & (RPMTRANS_FLAG_BUILD_PROBS|RPMTRANS_FLAG_NOPRETRANS|
-	RPMTRANS_FLAG_NOTRIGGERUN))) {
-
+    if (!(rpmtsFlags(ts) & (RPMTRANS_FLAG_NOPRETRANS|RPMTRANS_FLAG_NOTRIGGERUN))) {
 	runFileTriggers(ts, NULL, RPMSENSE_TRIGGERUN,
 			RPMSCRIPT_TRANSFILETRIGGER, 0);
 	runTransScripts(ts, PKG_TRANSFILETRIGGERUN);
     }
 
-    /* Actually install and remove packages, get final exit code */
-    rc = rpmtsProcess(ts) ? -1 : 0;
+    /* Actually install and remove packages */
+    nfailed = rpmtsProcess(ts);
 
     /* Run %posttrans scripts unless disabled */
     if (!(rpmtsFlags(ts) & (RPMTRANS_FLAG_NOPOSTTRANS))) {
@@ -1726,12 +1861,18 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     if (!(rpmtsFlags(ts) & (RPMTRANS_FLAG_NOPOSTTRANS|RPMTRANS_FLAG_NOTRIGGERIN))) {
 	runTransScripts(ts, PKG_TRANSFILETRIGGERIN);
     }
+    /* Final exit code */
+    rc = nfailed ? -1 : 0;
+
 exit:
     /* Run post transaction hook for all plugins */
     if (TsmPreDone) /* If TsmPre hook has been called, call the TsmPost hook */
 	rpmpluginsCallTsmPost(rpmtsPlugins(ts), ts, rc);
 
     /* Finish up... */
+    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_TEST) && nfailed >= 0) {
+	rpmtsSync(ts);
+    }
     (void) umask(oldmask);
     (void) rpmtsFinish(ts);
     rpmpsFree(tsprobs);
