@@ -6,8 +6,7 @@
 
 #include "header-py.h"
 #include "rpmds-py.h"
-
-#include "debug.h"
+#include "rpmstrpool-py.h"
 
 struct rpmdsObject_s {
     PyObject_HEAD
@@ -32,19 +31,19 @@ rpmds_Ix(rpmdsObject * s)
 static PyObject *
 rpmds_DNEVR(rpmdsObject * s)
 {
-    return Py_BuildValue("s", rpmdsDNEVR(s->ds));
+    return utf8FromString(rpmdsDNEVR(s->ds));
 }
 
 static PyObject *
 rpmds_N(rpmdsObject * s)
 {
-    return Py_BuildValue("s", rpmdsN(s->ds));
+    return utf8FromString(rpmdsN(s->ds));
 }
 
 static PyObject *
 rpmds_EVR(rpmdsObject * s)
 {
-    return Py_BuildValue("s", rpmdsEVR(s->ds));
+    return utf8FromString(rpmdsEVR(s->ds));
 }
 
 static PyObject *
@@ -53,12 +52,6 @@ rpmds_Flags(rpmdsObject * s)
     return Py_BuildValue("i", rpmdsFlags(s->ds));
 }
 
-static PyObject *
-rpmds_BT(rpmdsObject * s)
-{
-    return Py_BuildValue("i", (int) rpmdsBT(s->ds));
-}
-
 static PyObject *
 rpmds_TagN(rpmdsObject * s)
 {
@@ -72,9 +65,21 @@ rpmds_Color(rpmdsObject * s)
 }
 
 static PyObject *
-rpmds_Refs(rpmdsObject * s)
+rpmds_IsWeak(rpmdsObject * s)
 {
-    return Py_BuildValue("i", rpmdsRefs(s->ds));
+    return PyBool_FromLong(rpmdsIsWeak(s->ds));
+}
+
+static PyObject *
+rpmds_IsRich(rpmdsObject * s)
+{
+    return PyBool_FromLong(rpmdsIsRich(s->ds));
+}
+
+static PyObject *
+rpmds_IsReverse(rpmdsObject * s)
+{
+    return PyBool_FromLong(rpmdsIsReverse(s->ds));
 }
 
 static PyObject *
@@ -90,12 +95,7 @@ rpmds_iternext(rpmdsObject * s)
 
     /* If more to do, return a (N, EVR, Flags) tuple. */
     if (rpmdsNext(s->ds) >= 0) {
-	const char * N = rpmdsN(s->ds);
-	const char * EVR = rpmdsEVR(s->ds);
-	rpmTag tagN = rpmdsTagN(s->ds);
-	rpmsenseFlags Flags = rpmdsFlags(s->ds);
-
-	result = rpmds_Wrap(Py_TYPE(s), rpmdsSingle(tagN, N, EVR, Flags) );
+	result = rpmds_Wrap(Py_TYPE(s), rpmdsCurrent(s->ds));
     } else
 	s->active = 0;
 
@@ -115,21 +115,6 @@ rpmds_SetNoPromote(rpmdsObject * s, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", rpmdsSetNoPromote(s->ds, nopromote));
 }
 
-static PyObject *
-rpmds_Notify(rpmdsObject * s, PyObject * args, PyObject * kwds)
-{
-    const char * where;
-    int rc;
-    char * kwlist[] = {"location", "returnCode", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "si:Notify", kwlist,
-	    &where, &rc))
-	return NULL;
-
-    rpmdsNotify(s->ds, where, rc);
-    Py_RETURN_NONE;
-}
-
 /* XXX rpmdsFind uses bsearch on s->ds, so a sort is needed. */
 static PyObject *
 rpmds_Sort(rpmdsObject * s)
@@ -183,13 +168,23 @@ static PyObject *rpmds_Compare(rpmdsObject * s, PyObject * o)
     return PyBool_FromLong(rpmdsCompare(s->ds, ods->ds));
 }
 
-static PyObject * rpmds_Rpmlib(rpmdsObject * s)
+static PyObject *rpmds_Instance(rpmdsObject * s)
 {
+    return Py_BuildValue("i", rpmdsInstance(s->ds));
+}
+
+static PyObject * rpmds_Rpmlib(rpmdsObject * s, PyObject *args, PyObject *kwds)
+{
+    rpmstrPool pool = NULL;
     rpmds ds = NULL;
-    int xx;
+    char * kwlist[] = {"pool", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&:rpmds_Rpmlib", kwlist, 
+		 &poolFromPyObject, &pool))
+	return NULL;
 
     /* XXX check return code, permit arg (NULL uses system default). */
-    xx = rpmdsRpmlib(&ds, NULL);
+    rpmdsRpmlibPool(pool, &ds, NULL);
 
     return rpmds_Wrap(&rpmds_Type, ds);
 }
@@ -198,41 +193,45 @@ static struct PyMethodDef rpmds_methods[] = {
  {"Count",	(PyCFunction)rpmds_Count,	METH_NOARGS,
 	"Deprecated, use len(ds) instead.\n" },
  {"Ix",		(PyCFunction)rpmds_Ix,		METH_NOARGS,
-	"ds.Ix -> Ix		- Return current element index.\n" },
+	"ds.Ix -> Ix -- Return current element index.\n" },
  {"DNEVR",	(PyCFunction)rpmds_DNEVR,	METH_NOARGS,
-	"ds.DNEVR -> DNEVR	- Return current DNEVR.\n" },
+	"ds.DNEVR -> DNEVR -- Return current DNEVR.\n" },
  {"N",		(PyCFunction)rpmds_N,		METH_NOARGS,
-	"ds.N -> N		- Return current N.\n" },
+	"ds.N -> N -- Return current N.\n" },
  {"EVR",	(PyCFunction)rpmds_EVR,		METH_NOARGS,
-	"ds.EVR -> EVR		- Return current EVR.\n" },
+	"ds.EVR -> EVR -- Return current EVR.\n" },
  {"Flags",	(PyCFunction)rpmds_Flags,	METH_NOARGS,
-	"ds.Flags -> Flags	- Return current Flags.\n" },
- {"BT",		(PyCFunction)rpmds_BT,		METH_NOARGS,
-	"ds.BT -> BT	- Return build time.\n" },
+	"ds.Flags -> Flags -- Return current Flags.\n" },
  {"TagN",	(PyCFunction)rpmds_TagN,	METH_NOARGS,
-	"ds.TagN -> TagN	- Return current TagN.\n" },
+  "ds.TagN -> TagN -- Return TagN (RPMTAG_*NAME)\n\n"
+  "the type of all dependencies in this set.\n" },
  {"Color",	(PyCFunction)rpmds_Color,	METH_NOARGS,
-	"ds.Color -> Color	- Return current Color.\n" },
- {"Refs",	(PyCFunction)rpmds_Refs,	METH_NOARGS,
-	"ds.Refs -> Refs	- Return current Refs.\n" },
+	"ds.Color -> Color -- Return current Color.\n" },
  {"SetNoPromote",(PyCFunction)rpmds_SetNoPromote, METH_VARARGS|METH_KEYWORDS,
-	NULL},
- {"Notify",	(PyCFunction)rpmds_Notify,	METH_VARARGS|METH_KEYWORDS,
-	NULL},
+  "ds.SetNoPromote(noPromote) -- Set noPromote for this instance.\n\n"
+  "If True non existing epochs are no longer equal to an epoch of 0."},
  {"Sort",	(PyCFunction)rpmds_Sort,	METH_NOARGS,
 	NULL},
  {"Find",	(PyCFunction)rpmds_Find,	METH_O,
-	NULL},
+  "ds.find(other_ds) -- Return index of other_ds in ds"},
  {"Merge",	(PyCFunction)rpmds_Merge,	METH_O,
 	NULL},
  {"Search",     (PyCFunction)rpmds_Search,      METH_O,
 "ds.Search(element) -> matching ds index (-1 on failure)\n\
-- Check that element dependency range overlaps some member of ds.\n\
-The current index in ds is positioned at overlapping member upon success.\n" },
- {"Rpmlib",     (PyCFunction)rpmds_Rpmlib,      METH_NOARGS|METH_STATIC,
-	"ds.Rpmlib -> nds       - Return internal rpmlib dependency set.\n"},
+Check that element dependency range overlaps some member of ds.\n\
+The current index in ds is positioned at overlapping member." },
+ {"Rpmlib",     (PyCFunction)rpmds_Rpmlib,      METH_VARARGS|METH_KEYWORDS|METH_STATIC,
+	"ds.Rpmlib -> nds -- Return internal rpmlib dependency set.\n"},
  {"Compare",	(PyCFunction)rpmds_Compare,	METH_O,
-	NULL},
+  "ds.compare(other) -- Compare current entries of self and other.\n\nReturns True if the entries match each other, False otherwise"},
+ {"Instance",	(PyCFunction)rpmds_Instance,	METH_NOARGS,
+  "ds.Instance() -- Return rpmdb key of corresponding package or 0."},
+ {"IsWeak",	(PyCFunction)rpmds_IsWeak,	METH_NOARGS,
+  "ds.IsWeak() -- Return whether the dependency is weak."},
+ {"IsRich",	(PyCFunction)rpmds_IsRich,	METH_NOARGS,
+  "ds.IsRich() -- Return whether the dependency is rich."},
+ {"IsReverse",	(PyCFunction)rpmds_IsReverse,	METH_NOARGS,
+  "ds.IsReverse() -- Return whether the dependency is reversed."},
  {NULL,		NULL}		/* sentinel */
 };
 
@@ -255,14 +254,14 @@ rpmds_subscript(rpmdsObject * s, PyObject * key)
 {
     int ix;
 
-    if (!PyInt_Check(key)) {
+    if (!PyLong_Check(key)) {
 	PyErr_SetString(PyExc_TypeError, "integer expected");
 	return NULL;
     }
 
-    ix = (int) PyInt_AsLong(key);
+    ix = (int) PyLong_AsLong(key);
     rpmdsSetIx(s->ds, ix);
-    return Py_BuildValue("s", rpmdsDNEVR(s->ds));
+    return utf8FromString(rpmdsDNEVR(s->ds));
 }
 
 static PyMappingMethods rpmds_as_mapping = {
@@ -277,30 +276,75 @@ static int rpmds_init(rpmdsObject * s, PyObject *args, PyObject *kwds)
     return 0;
 }
 
+static int depflags(PyObject *o, rpmsenseFlags *senseFlags)
+{
+    int ok = 0;
+    PyObject *str = NULL;
+    rpmsenseFlags flags = RPMSENSE_ANY;
+
+    if (PyLong_Check(o)) {
+	ok = 1;
+	flags = PyLong_AsLong(o);
+    } else if (utf8FromPyObject(o, &str)) {
+	ok = 1;
+	for (const char *s = PyBytes_AsString(str); *s; s++) {
+	    switch (*s) {
+	    case '=':
+		flags |= RPMSENSE_EQUAL;
+		break;
+	    case '<':
+		flags |= RPMSENSE_LESS;
+		break;
+	    case '>':
+		flags |= RPMSENSE_GREATER;
+		break;
+	    default:
+		ok = 0;
+		break;
+	    }
+	}
+	Py_DECREF(str);
+    }
+
+    if (flags == (RPMSENSE_EQUAL|RPMSENSE_LESS|RPMSENSE_GREATER))
+	ok = 0;
+
+    if (ok)
+	*senseFlags = flags;
+
+    return ok;
+}
+
 static PyObject * rpmds_new(PyTypeObject * subtype, PyObject *args, PyObject *kwds)
 {
     PyObject *obj;
-    rpmTag tagN = RPMTAG_REQUIRENAME;
+    rpmTagVal tagN = RPMTAG_REQUIRENAME;
     rpmds ds = NULL;
     Header h = NULL;
-    char * kwlist[] = {"obj", "tag", NULL};
+    rpmstrPool pool = NULL;
+    char * kwlist[] = {"obj", "tag", "pool", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO&:rpmds_new", kwlist, 
-	    	 &obj, tagNumFromPyObject, &tagN))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO&|O&:rpmds_new", kwlist, 
+	    	 &obj, tagNumFromPyObject, &tagN,
+		 &poolFromPyObject, &pool))
 	return NULL;
 
     if (PyTuple_Check(obj)) {
 	const char *name = NULL;
 	const char *evr = NULL;
 	rpmsenseFlags flags = RPMSENSE_ANY;
-	if (PyArg_ParseTuple(obj, "s|is", &name, &flags, &evr)) {
-	    ds = rpmdsSingle(tagN, name, evr, flags);
+	/* TODO: if flags are specified, evr should be required too */
+	if (PyArg_ParseTuple(obj, "s|O&s", &name, depflags, &flags, &evr)) {
+	    ds = rpmdsSinglePool(pool, tagN, name, evr, flags);
+	} else {
+	    PyErr_SetString(PyExc_ValueError, "invalid dependency tuple");
+	    return NULL;
 	}
     } else if (hdrFromPyObject(obj, &h)) {
 	if (tagN == RPMTAG_NEVR) {
-	    ds = rpmdsThis(h, RPMTAG_PROVIDENAME, RPMSENSE_EQUAL);
+	    ds = rpmdsThisPool(pool, h, RPMTAG_PROVIDENAME, RPMSENSE_EQUAL);
 	} else {
-	    ds = rpmdsNew(h, tagN, 0);
+	    ds = rpmdsNewPool(pool, h, tagN, 0);
 	}
     } else {
 	PyErr_SetString(PyExc_TypeError, "header or tuple expected");
@@ -311,7 +355,11 @@ static PyObject * rpmds_new(PyTypeObject * subtype, PyObject *args, PyObject *kw
 }
 
 static char rpmds_doc[] =
-"";
+    "rpm.ds (dependendcy set) gives a more convenient access to dependencies\n\n"
+    "It can hold multiple entries of Name Flags and EVR.\n"
+    "It typically represents all dependencies of one kind of a package\n"
+    "e.g. all Requires or all Conflicts.\n"
+    ;
 
 PyTypeObject rpmds_Type = {
 	PyVarObject_HEAD_INIT(&PyType_Type, 0)
