@@ -43,29 +43,6 @@ char * stpncpy(char * dest, const char * src, size_t n);
 #define	getenv(_s)	__secure_getenv(_s)
 #endif
 
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#else
-#include <sys/file.h>
-#endif
-
-#ifdef HAVE_DIRENT_H
-# include <dirent.h>
-# define NLENGTH(direct) (strlen((direct)->d_name))
-#else /* not HAVE_DIRENT_H */
-# define dirent direct
-# define NLENGTH(direct) ((direct)->d_namlen)
-# ifdef HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif /* HAVE_SYS_NDIR_H */
-# ifdef HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif /* HAVE_SYS_DIR_H */
-# ifdef HAVE_NDIR_H
-#  include <ndir.h>
-# endif /* HAVE_NDIR_H */
-#endif /* HAVE_DIRENT_H */
-
 #if HAVE_LIMITS_H
 #include <limits.h>
 #endif
@@ -125,6 +102,4 @@ extern int fdatasync(int fildes);
 
 #include "misc/fnmatch.h"
 
-#include <dlfcn.h>
-
 #endif	/* H_SYSTEM */
